require 'spec_helper'

describe :pe_is_meep_master do
  let(:layout) { 'mono' }

  include_context 'meep_config'
  include_context 'override_trusted_information'

  context 'on the meep master' do
    let(:node) { 'master.rspec' }

    it { is_expected.to run.with_params.and_return(true) }
    it { is_expected.to run.with_params('master.rspec').and_return(true) }
    it { is_expected.to run.with_params('compile.rspec').and_return(false) }
    it do
      is_expected.to run.with_params('one', 'two').and_raise_error(ArgumentError, /expects at most 1 argument, got 2/)
    end
  end

  context 'on another node' do
    let(:node) { 'compile.rspec' }

    it { is_expected.to run.with_params.and_return(false) }
    it { is_expected.to run.with_params('master.rspec').and_return(true) }
  end

  context 'with broken pe.conf' do
    let(:layout) { 'broken.tmp' }
    let(:node) { 'master.rspec' }

    let(:hieradata_empty) do
      <<-HIERADATA
"node_roles": {
  "pe_role::monolithic::primary_master": []
}
      HIERADATA
    end

    let(:hieradata_missing) do
      <<-HIERADATA
"node_roles": {
}
      HIERADATA
    end

    it "returns false if no primary master is set" do
      run_with_custom_hieradata(layout, hieradata_empty) do
        is_expected.to run.with_params.and_return(false)
      end
    end

    it "returns false if primary master role is missing" do
      run_with_custom_hieradata(layout, hieradata_missing) do
        is_expected.to run.with_params.and_return(false)
      end
    end
  end
end
