require 'spec_helper'

describe :pe_is_node_a do
  let(:node) { 'master.rspec' }

  include_context 'meep_config'
  include_context 'override_trusted_information'

  context "monolithic" do
    let(:layout) { 'mono' }

    context "argument validation" do
      it "requires at least one arg" do
        is_expected.to run.with_params.and_raise_error(ArgumentError, /expects between 1 and 2.*got none/i)
      end

      it "checks that component argument only accepts strings" do
        is_expected.to run.with_params(1).and_raise_error(ArgumentError, /component.*expects a match for Component/i)
      end

      it "raises an error for an unknown component type" do
        is_expected.to run.with_params('notacomponent').and_raise_error(ArgumentError, /component.*expects a match for Component/)
      end

      it "checks that certname argument only accepts strings" do
        is_expected.to run.with_params('primary_master', 1).and_raise_error(ArgumentError, /certname.*expects a String/i)
      end

      it "does not accept three args" do
        is_expected.to run.with_params('foo','bar','baz').and_raise_error(ArgumentError, /expects between 1 and 2.*got 3/i)
      end

    end

    it { is_expected.to run.with_params('primary_master').and_return(true) }
    it { is_expected.to run.with_params('console').and_return(true) }
    it { is_expected.to run.with_params('puppetdb').and_return(true) }
    it { is_expected.to run.with_params('database').and_return(true) }
    it { is_expected.to run.with_params('orchestrator').and_return(true) }
    it { is_expected.to run.with_params('primary_master_replica').and_return(false) }
    it { is_expected.to run.with_params('enabled_primary_master_replica').and_return(false) }
  end

  context "split" do
    let(:layout) { 'split' }

    it { is_expected.to run.with_params('primary_master').and_return(true) }
    it { is_expected.to run.with_params('console').and_return(false) }
    it { is_expected.to run.with_params('puppetdb').and_return(false) }
    it { is_expected.to run.with_params('database').and_return(false) }
    it { is_expected.to run.with_params('orchestrator').and_return(true) }
    it { is_expected.to run.with_params('primary_master_replica').and_return(false) }
    it { is_expected.to run.with_params('enabled_primary_master_replica').and_return(false) }

    context "providing certname" do
      it { is_expected.to run.with_params('primary_master', 'notappearing.inthis.conf').and_return(false) }
      it { is_expected.to run.with_params('console', 'console.rspec').and_return(true) }
      it { is_expected.to run.with_params('puppetdb', 'puppetdb1.rspec').and_return(true) }
      it { is_expected.to run.with_params('database', 'puppetdb2.rspec').and_return(true) }
      it { is_expected.to run.with_params('orchestrator', 'console.rspec').and_return(false) }
      it { is_expected.to run.with_params('primary_master_replica', 'console.rspec').and_return(false) }
      it { is_expected.to run.with_params('enabled_primary_master_replica', 'console.rspec').and_return(false) }
    end
  end

  context "replica" do
    let(:layout) { 'replica' }

    it { is_expected.to run.with_params('primary_master').and_return(true) }
    it { is_expected.to run.with_params('console').and_return(true) }
    it { is_expected.to run.with_params('puppetdb').and_return(true) }
    it { is_expected.to run.with_params('database').and_return(true) }
    it { is_expected.to run.with_params('orchestrator').and_return(true) }
    it { is_expected.to run.with_params('primary_master_replica').and_return(false) }
    it { is_expected.to run.with_params('enabled_primary_master_replica').and_return(false) }

    context "providing certname" do
      context "of a provisioned replica" do
        # Currently the function does not consider the components of a provisioned replica
        it { is_expected.to run.with_params('primary_master', 'provisioned_replica.conf').and_return(false) }
        it { is_expected.to run.with_params('console', 'provisioned_replica.rspec').and_return(false) }
        it { is_expected.to run.with_params('puppetdb', 'provisioned_replica.rspec').and_return(false) }
        it { is_expected.to run.with_params('database', 'provisioned_replica.rspec').and_return(false) }
        # The orchestrator is not installed on a replica
        it { is_expected.to run.with_params('orchestrator', 'provisioned_replica.rspec').and_return(false) }

        it { is_expected.to run.with_params('primary_master_replica', 'provisioned_replica.rspec').and_return(true) }
        it { is_expected.to run.with_params('enabled_primary_master_replica', 'provisioned_replica.rspec').and_return(false) }
      end

      context "of an enabled replica" do
        # Currently the function does not consider the components of an enabled replica
        it { is_expected.to run.with_params('primary_master', 'enabled_replica.rspec').and_return(false) }
        it { is_expected.to run.with_params('console', 'enabled_replica.rspec').and_return(false) }
        it { is_expected.to run.with_params('puppetdb', 'enabled_replica.rspec').and_return(false) }
        it { is_expected.to run.with_params('database', 'enabled_replica.rspec').and_return(false) }
        # The orchestrator is not installed on a replica
        it { is_expected.to run.with_params('orchestrator', 'enabled_replica.rspec').and_return(false) }

        it { is_expected.to run.with_params('primary_master_replica', 'enabled_replica.rspec').and_return(true) }
        it { is_expected.to run.with_params('enabled_primary_master_replica', 'enabled_replica.rspec').and_return(true) }
      end
    end
  end
end
