require 'spec_helper'

describe :pe_meep_master do
  let(:node) { 'master.rspec' }

  include_context 'meep_config'
  include_context 'override_trusted_information'

  context 'monolithic' do
    let(:layout) { 'mono' }

    it { is_expected.to run.with_params.and_return('master.rspec') }
  end

  context 'split' do
    let(:layout) { 'split' }

    it { is_expected.to run.with_params.and_return('master.rspec') }
  end

  context 'broken pe.conf' do
    let(:layout) { 'broken.tmp' }
    let(:hieradata_empty) do
      <<-HIERADATA
"node_roles": {
  "pe_role::monolithic::primary_master": []
}
      HIERADATA
    end

    let(:hieradata_missing) do
      <<-HIERADATA
"node_roles": {
}
      HIERADATA
    end

    it "returns nil if no primary master is set" do
      run_with_custom_hieradata(layout, hieradata_empty) do
        is_expected.to run.with_params.and_return(nil)
      end
    end

    it "returns nil if primary master role is missing" do
      run_with_custom_hieradata(layout, hieradata_missing) do
        is_expected.to run.with_params.and_return(nil)
      end
    end
  end

  context 'default pe.conf with trusted.certname' do
    let(:layout) { 'default' }
    let(:node) { 'master.rspec' }
    let(:hieradata) do
      <<-HIERADATA
        "node_roles": {
          "pe_role::monolithic::primary_master": ["%{::trusted.certname}"]
        }
      HIERADATA
    end

    it "find the primary master based on trusted.certname value" do
      run_with_custom_hieradata(layout, hieradata) do
        is_expected.to run.with_params.and_return('master.rspec')
      end
    end
  end
end
