require 'spec_helper'

describe :pe_services_list do
  let(:node) { 'master.rspec' }

  include_context 'meep_config'
  include_context 'override_trusted_information'

  context "monolithic" do
    let(:layout) { 'mono' }
    let(:mono_services) do
      {
        "primary" => {
          "puppetserver" => [
            {
              'node_certname' => 'master.rspec',
              'port'          => 8140
            }
          ],
          "puppetdb"     => [
            {
              'node_certname' => 'master.rspec',
              'port'          => 8081
            }
          ],
          "classifier"      => [
            {
              'node_certname' => 'master.rspec',
              'port'          => 4433
            }
          ],
          "activity"      => [
            {
              'node_certname' => 'master.rspec',
              'port'          => 4433
            }
          ],
          "rbac"      => [
            {
              'node_certname' => 'master.rspec',
              'port'          => 4433
            }
          ],
          "orchestrator" => [
            {
              'node_certname' => 'master.rspec',
              'port'          => 8143
            }
          ],
          "pcp-broker" => [
            {
              'node_certname' => 'master.rspec',
              'port'          => 8142
            }
          ],
        },
        "secondary" => {
          "puppetserver" => [
            {
              'node_certname' => 'compile.rspec',
              'port'          => 8140
            }
          ],
        },
        "replica" => {},
      }
    end

    context "validating arguments" do
      it do
        is_expected.to run.with_params('foo', 'bar')
          .and_raise_error(ArgumentError, /expects no arguments.*got 2/i)
      end
    end

    it 'lists monolithic services' do
      services_list = subject.execute()

      mono_services.each do |type, services|
        services.each do |service, info|
          expect(services_list[type][service].first).to include(info.first)
        end
      end
    end
  end

  context "split" do
    let(:layout) { 'split' }
    let(:split_services) do
      {
        "primary" => {
          "puppetserver" => [
            {
              'node_certname' => 'master.rspec',
              'port'          => 8140
            }
          ],
          "puppetdb"     => [
            {
              'node_certname' => 'puppetdb1.rspec',
              'port'          => 8081
            },
            {
              'node_certname' => 'puppetdb2.rspec',
              'port'          => 8081
            },
          ],
          "classifier"    => [
            {
              'node_certname' => 'console.rspec',
              'port'          => 4433
            }
          ],
          "activity"    => [
            {
              'node_certname' => 'console.rspec',
              'port'          => 4433
            }
          ],
          "rbac"    => [
            {
              'node_certname' => 'console.rspec',
              'port'          => 4433
            }
          ],
          "orchestrator" => [
            {
              'node_certname' => 'master.rspec',
              'port'          => 8143
            }
          ],
          "pcp-broker" => [
            {
              'node_certname' => 'master.rspec',
              'port'          => 8142
            }
          ],
        },
        "secondary" => {
          "puppetserver" => [
            {
              'node_certname' => 'compile.rspec',
              'port'          => 8140
            }
          ],
        },
        "replica" => {},
      }
    end

    it 'lists split services' do
      services_list = subject.execute()

      split_services.each do |type, services|
        services.each do |service, info|
          expect(services_list[type][service].first).to include(info.first)
        end
      end
    end
  end

  context "replica" do
    let(:layout) { 'replica' }
    let(:replica_services) do
      {
        "primary" => {
          "puppetserver" => [
            {
              'node_certname' => 'master.rspec',
              'port'          => 8140
            },
          ],
          "puppetdb"     => [
            {
              'node_certname' => 'master.rspec',
              'port'          => 8081
            },
          ],
          "classifier"      => [
            {
              'node_certname' => 'master.rspec',
              'port'          => 4433
            },
          ],
          "rbac"      => [
            {
              'node_certname' => 'master.rspec',
              'port'          => 4433
            },
          ],
          "activity"      => [
            {
              'node_certname' => 'master.rspec',
              'port'          => 4433
            },
          ],
          "orchestrator" => [
            {
              'node_certname' => 'master.rspec',
              'port'          => 8143
            },
          ],
          "pcp-broker" => [
            {
              'node_certname' => 'master.rspec',
              'port'          => 8142
            }
          ],
        },
        "secondary" => {
          "puppetserver" => [
            {
              'node_certname' => 'compile.rspec',
              'port'          => 8140
            }
          ],
        },
        "replica" => {
          "puppetserver" => [
            {
              'node_certname' => 'provisioned_replica.rspec',
              'port'          => 8140
            },
            {
              'node_certname' => 'enabled_replica.rspec',
              'port'          => 8140
            },
          ],
          "puppetdb"     => [
            {
              'node_certname' => 'provisioned_replica.rspec',
              'port'          => 8081
            },
            {
              'node_certname' => 'enabled_replica.rspec',
              'port'          => 8081
            },
          ],
          "classifier"      => [
            {
              'node_certname' => 'provisioned_replica.rspec',
              'port'          => 4433
            },
            {
              'node_certname' => 'enabled_replica.rspec',
              'port'          => 4433
            },
          ],
          "rbac"      => [
            {
              'node_certname' => 'provisioned_replica.rspec',
              'port'          => 4433
            },
            {
              'node_certname' => 'enabled_replica.rspec',
              'port'          => 4433
            },
          ],
          "activity"      => [
            {
              'node_certname' => 'provisioned_replica.rspec',
              'port'          => 4433
            },
            {
              'node_certname' => 'enabled_replica.rspec',
              'port'          => 4433
            },
          ],
        },
      }
    end

    # This construction is necessary because of the need to compose match_array
    # inside the hash.
    it 'lists replica services' do
      services_list = subject.execute()

      replica_services.each do |type, services|
        services.each do |service, info|
          info.each do |host_info|
            expect(services_list[type][service]).to include(include(host_info))
          end
        end
      end
    end
  end

  context 'overrides' do
    let(:layout) { 'overrides' }
    let(:overrides_services) do
      {
        "primary" => {
          "puppetserver" => [
            include({
              'node_certname' => 'master.rspec',
              'port'          => 8140
            })
          ],
          "puppetdb"     => [
            include({
              'node_certname' => 'puppetdb1.rspec',
              'port'          => 8081
            }),
            include({
              'node_certname' => 'puppetdb2.rspec',
              'port'          => 5678
            }),
          ],
          "rbac"    => [
            include({
              'node_certname' => 'console.rspec',
              'port'          => 1234
            }),
          ],
          "activity"    => [
            include({
              'node_certname' => 'console.rspec',
              'port'          => 1234
            })
          ],
          "classifier"    => [
            include({
              'node_certname' => 'console.rspec',
              'port'          => 1234
            })
          ],
          "orchestrator" => [
            include({
              'node_certname' => 'master.rspec',
              'port'          => 8143
            })
          ],
          "pcp-broker" => [
            include({
              'node_certname' => 'master.rspec',
              'port'          => 8142
            })
          ],
        },
        "secondary" => {},
        "replica" => {},
      }
    end

    it 'lists overrides services' do
      services_list = subject.execute()

      overrides_services.each do |type, services|
        services.each do |service, info|
          expect(services_list[type][service]).to match(info)
        end
      end
    end
  end
end
