# Providers are initialized when rspec-puppet checks dependencies by
# transforming the generated catalog.to_ral. However there is no available
# provider for scheduled_task on non-Windows platforms that we spec test on.
# Similarly the windows File provider runs into issues with missing gems and
# non-existent capabilites, particularly around permissions (supports_acl?()
# calls).
#
# Fortunately we don't care about provider behavior, we're just testing
# dependencies in the relationship_graph.
RSpec.shared_context 'windows' do
  Puppet::Type.type(:file).provide(:stub_windows) do
    confine    :operatingsystem => :windows
  end
  # This stub class avoids errors coming back when the type attempts to
  # validate the trigger parameter with the non-existent provider.
  Puppet::Type.type(:scheduled_task).provide(:stub_task_scheduler) do
    confine    :operatingsystem => :windows
    def validate_trigger(value); end
  end
end

RSpec.shared_context('meep run') do
  around(:each) do |example|
    begin
      PuppetX::Puppetlabs::Meep::Config.local_meep_run_in_progress!
      example.run
    ensure
      PuppetX::Puppetlabs::Meep::Config.send(:remove_instance_variable, :@meep_run_in_progress)
    end
  end
end

RSpec.shared_context('bootstrapping') do
  around(:each) do |example|
    begin
      PuppetX::Puppetlabs::Meep::Config.bootstrap_run_in_progress!(pe_version)
      example.run
    ensure
      PuppetX::Puppetlabs::Meep::Config.send(:remove_instance_variable, :@bootstrap_version)
    end
  end
end

RSpec.shared_context('Config.local prep') do
  let(:facts) do
    {
      "operatingsystem" => "CentOS",
      "platform_tag" => "el-7-x86_64",
    }
  end

  around(:each) do |example|
    begin
      # So that a correct TrustedInformation.local(node) is created here:
      # https://github.com/puppetlabs/puppet/blob/4.9.4/lib/puppet/parser/compiler.rb#L825-L827
      # Rather than using the test_helper's generic one:
      # https://github.com/puppetlabs/puppet/blob/4.9.4/lib/puppet/test/test_helper.rb#L137-L142
      Puppet.ignore(:trusted_information)

      certname = Puppet[:certname]
      Puppet[:certname] = node
      example.run
    ensure
      Puppet.restore(:trusted_information)
      Puppet[:certname] = certname
    end
  end

  before(:each) do
    # Keeps us from hitting local system for facts when generating Puppet::Node.
    # Cannot be inside an around block (rspec limitation with rspec mocks).
    allow(Facter).to receive(:to_hash).and_return(facts)
  end
end
