require 'spec_helper'

# This is a reduced version of ruby_spec.rb just to ensure we can subclass as
# documented
$LOAD_PATH << './spec/fixtures/modules/inherit_pe_ini_setting/lib'

provider_class = Puppet::Type.type(:inherit_pe_ini_setting).provider(:pe_ini_setting)
describe provider_class do
  include PuppetlabsSpec::Files

  let(:tmpfile) { tmpfilename('inherit_pe_ini_setting_test') }

  def validate_file(expected_content, tmpfile )
    expect(File.read(tmpfile)).to eq(expected_content)
  end


  before :each do
    File.open(tmpfile, 'w') do |fh|
      fh.write(orig_content)
    end
  end

  context 'when calling instances' do
    let(:orig_content) { '' }

    it 'should parse nothing when the file is empty' do
      allow(described_class).to receive(:file_path).and_return(tmpfile)
      expect(described_class.instances).to eq([])
    end

    context 'when the file has contents' do
      let(:orig_content) {
        <<-EOS
# A comment
red = blue
green = purple
        EOS
      }

      it 'should parse the results' do
        allow(described_class).to receive(:file_path).and_return(tmpfile)
        instances = described_class.instances
        instances.size.should == 2
        # inherited version of namevar flattens the names
        names = instances.map do |instance|
          instance.instance_variable_get(:@property_hash)[:name]
        end
        names.sort.should == [ 'green', 'red' ]
      end
    end
  end

  context 'when ensuring that a setting is present' do
    let(:orig_content) { '' }

    it 'should add a value to the file' do
      allow(described_class).to receive(:file_path).and_return(tmpfile)
      resource = Puppet::Type::Inherit_pe_ini_setting.new({
        :setting => 'set_this',
        :value   => 'to_that',
      })
      provider = described_class.new(resource)
      provider.create
      validate_file("set_this=to_that\n", tmpfile)
    end
  end
end
