#! /usr/bin/env ruby
require 'spec_helper'

pe_ini_setting = Puppet::Type.type(:pe_ini_setting)

describe pe_ini_setting do
  describe 'path validation' do
    subject { lambda { described_class.new(:name => 'foo', :path => path) } }

    context 'on posix platforms' do
      before(:each) { Puppet.features.stub(:posix?) { true } }
      before(:each) { Puppet.features.stub(:microsoft_windows?) { false } }
      before(:each) { Puppet::Util::Platform.stub(:windows?) { false } }

      context 'with an absolute path' do
        let(:path) { '/absolute/path' }
        it { should_not raise_exception }
      end

      context 'with a relative path' do
        let(:path) { 'relative/path' }
        it { should raise_exception }
      end
    end

    context 'on windows platforms' do
      before(:each) { Puppet.features.stub(:posix?) { false } }
      before(:each) { Puppet.features.stub(:microsoft_windows?) { true } }
      before(:each) { Puppet::Util::Platform.stub(:windows?) { true } }

      context 'with an absolute path with front slashes' do
        let(:path) { 'c:/absolute/path' }
        it { should_not raise_exception }
      end

      context 'with an absolute path with backslashes' do
        let(:path) { 'c:\absolute\path' }
        it { should_not raise_exception }
      end

      context 'with an absolute path with mixed slashes' do
        let(:path) { 'c:/absolute\path' }
        it { should_not raise_exception }
      end

      context 'with a relative path with front slashes' do
        let(:path) { 'relative/path' }
        it { should raise_exception }
      end

      context 'with a relative path with back slashes' do
        let(:path) { 'relative\path' }
        it { should raise_exception }
      end
    end
  end
end
