# Copies the currently installed postgresql binaries into a versioned
# namespace archive which can be used by pg_upgrade to migrate an older
# database to a newer one.
#
# We run the pg_upgrade command from the newly installed postgresql
# package, but it needs to use the binaries of the previous installation to
# process the old data format:
#
# * bin
# * lib
# * share
#
# These must be:
#
# * chown pe-postgres
#
# This class should be contained prior to puppet_enterprise::packages
# replacing the original postgresql files.
class pe_install::upgrade::postgres::backup_binaries(
  String $source_app_dir,
  String $archived_app_dir,
) {

  $source_bin_dir   = "${source_app_dir}/bin/"
  $source_lib_dir   = "${source_app_dir}/lib/"
  $source_share_dir = "${source_app_dir}/share/"

  $archived_bin_dir   = "${archived_app_dir}/bin"
  $archived_lib_dir   = "${archived_app_dir}/lib"
  $archived_share_dir = "${archived_app_dir}/share"

  File {
    owner => 'pe-postgres',
    group => 'pe-postgres',
    mode  => '0640',
  }

  $copy_postgres_params = {
    ensure             => directory,
    recurse            => true,
  }

  file { $archived_app_dir:
    ensure => directory,
    mode   => '0644',
  }
  file { $archived_bin_dir:
    mode   => '0750',
    source => $source_bin_dir,
    *      => $copy_postgres_params,
  }
  file { $archived_lib_dir:
    source => $source_lib_dir,
    *      => $copy_postgres_params,
  }
  file { $archived_share_dir:
    source => $source_share_dir,
    *      => $copy_postgres_params,
  }
}
