require 'spec_helper'

describe 'pe_install::install' do
  let(:hiera_config) { 'spec/fixtures/hiera/hiera.yaml' }

  context 'supported operating systems' do
    on_supported_os.each do |os, base_facts|
      context "on #{os}", :os => os do
        include_context('hosts', os, base_facts)

        let(:pre_condition) do
          <<-PE_INSTALL_STUB
# Stub out the main classes so we only generate the various is_* test variables.
class pe_install::validate {}
class pe_install::prepare::certificates {}
class pe_install::prepare {
  include pe_install::prepare::certificates
}
include pe_install
          PE_INSTALL_STUB
        end

        context 'monolithic install' do
          let(:node) { 'master.rspec' }

          it { is_expected.to compile.with_all_deps }
          it { is_expected.to contain_class('puppet_enterprise::profile::agent') }
          it { is_expected.to contain_service('puppet').with_ensure('running').with_enable(true) }
          it { is_expected.to contain_file('/opt/puppetlabs/bin/puppet-infrastructure') }
        end

        # All other profiles depend on pe_install::is_* variables which
        # aren't set since this is just an isolated 'include
        # pe_install::install' that is being tested
        #
        # See spec/hosts...
        #
        # TODO: although those values could be isolated using pre_condition()
        # (see spec/classes/prepare/certificates_spec.rb).  Alternately, we could
        # modify the manifests to explicitly pass all values the class needs.

        # Disable management of the puppet service using pe_infrastructure::agent::puppet_service_managed
        context 'puppet service management disabled' do
          # Modify node name so that rspec-puppet doesn't cache the hiera data
          let(:node) { 'nopuppetservice.master.rspec' }

          let(:pe_conf) do
            <<-HIERA
"puppet_enterprise::puppet_master_host": "#{node}"
"pe_infrastructure::agent::puppet_service_managed": "false"
              HIERA
          end

          around(:each) do |example|
            run_with_custom_pe_conf(node, pe_conf, example)
          end

          it { is_expected.not_to contain_service('puppet') }
        end
      end
    end
  end
end
