require 'spec_helper'

describe 'pe_install::upgrade::postgres' do
  on_filtered_os(:redhat).each do |os, base_facts|
    let(:node) { 'master.rspec' }

    include_context('hosts', os, base_facts)

    let(:context_facts) do
      {
        :pe_postgresql_info          => {
          'installed_server_version' => '9.4',
        }
      }
    end
    let(:params) do
      {
        :old_postgres_version => '9.4',
        :new_postgres_version => '9.6',
      }
    end

    def pre_condition
      <<-PE_INSTALL_STUB
# Stub out the main classes so we only generate the various is_* test variables.
class pe_install::validate {}
class pe_install::prepare {}
class pe_install::install {}
include pe_install
package { 'postgresql-client': }
package { 'postgresql-server': }
package { 'postgresql-contrib': }
# Stub out the postgresql classes that we have dependency tests on
class pe_postgresql::server::config {}
class pe_postgresql::server::initdb {}
class pe_postgresql::server::service {}
include pe_postgresql::server::config
include pe_postgresql::server::initdb
include pe_postgresql::server::service
# include puppet_enterprise::packages which we have a dependency on
include puppet_enterprise::packages
realize(Package['pe-postgresql96-pglogical'])
      PE_INSTALL_STUB
    end

    it { is_expected.to compile.with_all_deps }
    it { is_expected.to contain_file('/opt/puppetlabs/server/apps/postgresql/9.4') }

    it do
      is_expected.to contain_exec('copy postgresql.conf from old install location to new install location')
        .with_before(['Class[Pe_postgresql::Server::Config]', 'Pe_install::Stop_service[Stop pe-postgresql before pg_upgrade]'])
        .with_require(['Package[postgresql-server]','Class[Pe_postgresql::Server::Initdb]'])
    end

    it do
      is_expected.to contain_pe_install__stop_service('Ensure old pe-postgresql server stopped before upgrading package')
        .with_before_reference(['Package[postgresql-server]'])
    end

    it do
      is_expected.to contain_pe_install__stop_service('Stop pe-postgresql before pg_upgrade')
        .with_before_reference(['Exec[migrate to new postgres]'])
        .with_after_reference(['Package[postgresql-server]'])
    end

    it do
      is_expected.to contain_exec('migrate to new postgres')
        .with_command(/pg_upgrade/)
        .with_timeout(0)
        .with_before('Class[Pe_postgresql::Server::Config]')
        .with_require(['Package[postgresql-client]', 'Package[postgresql-server]', 'Package[postgresql-contrib]', 'Package[pe-postgresql96-pglogical]', 'Class[Pe_postgresql::Server::Initdb]'])
    end

    it do
      is_expected.to contain_class('puppet_enterprise::pg::pglogical::drop_orphaned_triggers')
        .with_require('Class[Pe_postgresql::Server::Service]')
        .with_before('Exec[vacuum the newly migrated database]')
    end

    it do
      is_expected.to contain_exec('vacuum the newly migrated database')
        .with_command(/vacuumdb/)
        .with_timeout(0)
        .with_require('Class[Pe_postgresql::Server::Service]')
    end

    context 'changing upgrade timeout' do
      before(:each) do
        params[:pg_upgrade_timeout] = 60
      end

      it do
        is_expected.to contain_exec('migrate to new postgres')
          .with_timeout(60)
      end
      it do
        is_expected.to contain_exec('vacuum the newly migrated database')
          .with_timeout(60)
      end
    end

    context 'with 9.6 installed' do
      before(:each) do
        facts[:pe_postgresql_info]['installed_server_version'] = '9.6'
      end

      it { is_expected.to_not contain_file('/opt/puppetlabs/server/apps/postgresql/9.4') }
      it { is_expected.to_not contain_exec('copy postgresql.conf from old install location to new install location') }
    end

    context 'skip pglogical' do
      before(:each) do
        params[:require_pglogical] = false
      end

      it do
        is_expected.to contain_exec('migrate to new postgres')
          .with_require(['Package[postgresql-client]', 'Package[postgresql-server]', 'Package[postgresql-contrib]', 'Class[Pe_postgresql::Server::Initdb]'])
      end

      it do
        is_expected.to_not contain_class('puppet_enterprise::pg::pglogical::drop_orphaned_triggers')
      end
    end
  end
end
