require 'spec_helper'

describe 'pe_install::upgrade::remove_logrotate_config_for_tk_services' do
  let(:hiera_config) { 'spec/fixtures/hiera/hiera.yaml' }

  context 'filtered operating systems' do
    on_filtered_os(:debian) do |os, base_facts|
      context "on #{os}" do
        let(:node) { 'master.rspec' }
        let(:layout) { 'mono' }
        let(:facts) do
          base_facts.merge(
            :layout => layout
          )
        end
        service_list = [
            'pe-console-services',
            'pe-orchestration-services',
            'pe-puppetdb',
            'pe-puppetserver'
        ]

        context 'with a pe_install stub' do
          def pre_condition
            <<-PE_INSTALL_STUB
  # Stub out the main classes so we only generate the various is_* test variables.
  class pe_install::validate {}
  class pe_install::prepare {}
  class pe_install::install {}
  include pe_install
            PE_INSTALL_STUB
          end

          describe 'upgrade_from is undef' do
            it { is_expected.to compile }
          end

          describe 'upgrading from a later release than 2016.5.0' do
            let(:params) do
              {
                :upgrade_from => '2017.2.0'
              }
            end
            service_list.each do |service|
              it { is_expected.to_not contain_file("/etc/logrotate.d/#{service}") }
            end
          end

          describe 'upgrading from earlier than 2016.5.0' do
            let(:params) do
              {
                :upgrade_from => '2016.4.6'
              }
            end
            service_list.each do |service|
              it { is_expected.to contain_file("/etc/logrotate.d/#{service}")
                                      .with_ensure('absent') }
            end
          end

          describe 'upgrading from 2016.5.0 to newer' do
            let(:params) do
              {
                :upgrade_from => '2016.5.0'
              }
            end
            service_list.each do |service|
              it { is_expected.to_not contain_file("/etc/logrotate.d/#{service}") }
            end
          end
        end
      end
    end
  end
end
