require 'spec_helper'

describe 'pe_install::upgrade::stop_services' do
  let(:hiera_config) { 'spec/fixtures/hiera/hiera.yaml' }

  context 'supported operating systems' do
    on_supported_os.each do |os, base_facts|
      context "on #{os}", :os => os do
        let(:node) { 'master.rspec' }
        let(:layout) { 'mono' }
        let(:facts) do
          base_facts.merge(
            :upgrade__stop_services => true, #bust rspec-puppet catalog cache
            :layout       => layout,
            :platform_tag => platform_tag_for(os, base_facts),
          ).merge(context_facts)
        end
        let(:context_facts) { {} }
        let(:params) do
          {
            :force_service_halt => false,
          }
        end

        default_ref = ['Class[Puppet_enterprise::Packages]', 'Package[postgresql-client]', 'Package[postgresql-server]', 'Package[postgresql-contrib]', 'Package[pe-nginx]']
        service_list = [
            ['pe-nginx', default_ref],
            ['pe-console-services', default_ref],
            ['pe-puppetdb', default_ref],
            ['pe-postgresql', default_ref ],
            ['pe-orchestration-services', default_ref],
            ['pe-puppetserver', default_ref],
            ['pe-bolt-server', default_ref],
            ['pe-ace-server', default_ref],
        ]

        it { is_expected.to compile.with_all_deps }

        context 'with a pe_install stub' do
          def pre_condition
            <<-PE_INSTALL_STUB
# Stub out the main classes so we only generate the various is_* test variables.
class pe_install::validate {}
class pe_install::prepare {}
class pe_install::install {}
include pe_install

# Explicit requires neede for Puppet 5 dependency checking
# (These references are managed by puppetlabs-pe_postgresql)
package { 'postgresql-client': }
package { 'postgresql-server': }
package { 'postgresql-contrib': }
package { 'pe-nginx': }
            PE_INSTALL_STUB
          end

          let(:pe_postgresql_info) do
            {
              'installed_server_version' => '9.6',
            }
          end
          let(:params) do
            {
              :force_service_halt => true,
            }
          end

          describe 'upgrading to 2016.2.0' do
            ['2015.3.0','2016.1.2', ].each do |upgrade_ver|
              context "from #{upgrade_ver}" do
                let(:context_facts) do
                  {
                    :pe_server_version => upgrade_ver,
                    :pe_postgresql_info => pe_postgresql_info,
                  }
                end

                service_list.each do |service, refs|
                  it do
                    is_expected.to contain_pe_install__stop_service("Stop #{service} service")
                      .with_before_reference(refs)
                  end
                end

                it 'kills pe-postgresql' do
                  before_list = default_ref.to_s.gsub('"','')
                  is_expected.to contain_service("Shutdown pe-postgresql before #{before_list}")
                    .with_stop('kill -INT `head -1 /opt/puppetlabs/server/data/postgresql/9.6/data/postmaster.pid` && while [ -e /opt/puppetlabs/server/data/postgresql/9.6/data/postmaster.pid ]; do sleep 1; done')
                end
              end
            end

            # pe-orchestration-services didn't exist in 2015.2.3
            context 'from 2015.2.3' do
              let(:context_facts) do
                {
                  :pe_server_version => '2015.2.3',
                  :pe_postgresql_info => pe_postgresql_info,
                }
              end

              service_list.each do |service, refs|
                next if service == 'pe-orchestration-services'
                it do
                  is_expected.to contain_pe_install__stop_service("Stop #{service} service")
                    .with_before_reference(refs)
                end
              end

              it { is_expected.to_not contain_pe_install__stop_service("Stop pe-orchestration-services service") }
            end

            context 'without pe_postgresql_info fact' do
              let(:context_facts) do
                {
                  :pe_server_version => '2015.2.3',
                }
              end

              it do
                expect { catalogue }.to(raise_error(Puppet::ParseError, /pe_postgresql_info fact is missing/))
              end
            end
          end
        end
      end
    end
  end
end
