require 'spec_helper'

RSpec.shared_examples 'a base run on the console node' do
  it { is_expected.to compile.with_all_deps }
  it { is_expected.to contain_class('puppet_enterprise::profile::console') }
  it { is_expected.to_not contain_class('puppet_enterprise::profile::master') }
  it { is_expected.to_not contain_class('puppet_enterprise::profile::puppetdb') }
  it { is_expected.to_not contain_class('puppet_enterprise::profile::database') }
  it { is_expected.to contain_class('puppet_enterprise::profile::agent') }
  it do
    is_expected.to(have_a_resource('Class[Puppet_enterprise::Packages]')
                   .that_is_a_dependency_for_downstream_resources(
                     'Class[Puppet_enterprise::Profile::Console]',
                     'Class[Puppet_enterprise::Profile::Agent]',
                     'File[/opt/puppetlabs/server/bin/set_console_admin_password.rb]',
                   ))
  end
  it { is_expected.to contain_package('postgresql-client')
        .with_ensure('latest')
        .with_name('pe-postgresql96') }
  it { is_expected.to contain_package('pe-puppet-enterprise-release') }
  it { is_expected.to contain_package('pe-console-services') }

  # The password string is shell escaped, so we must NOT quote it.
  it { is_expected.to contain_exec('set console admin password')
      .with_command(/puppet-infra console_password/) }
end

describe 'console host' do
  context 'supported operating systems' do
    on_filtered_os(:redhat, :debian, :sles) do |os, base_facts|
      context "on #{os}", :os => os do
        include_context('hosts', os, base_facts)
        include_context 'upgrade service shutdown package refs'
        let(:pg_refs) { [] }

        let(:node) { 'console.rspec' }
        let(:layout) { 'split' }

        include_examples 'a base run on the console node'
        include_examples 'no repo config during install or upgrade'

        it { is_expected.to_not have_a_resource("Service[Shutdown pe-console-services #{package_refs_string}]") }
        it { is_expected.to_not have_a_resource("Service[Shutdown pe-nginx #{package_refs_string}]") }

        context 'when upgrading' do
          let(:node) { 'consoleupgrade.rspec' }
          let(:layout) { 'none' }
          let(:pe_conf) do
            <<-HIERA
{
"puppet_enterprise::certificate_authority_host": "master.rspec"
"puppet_enterprise::puppet_master_host": "master.rspec"
"puppet_enterprise::console_host": "#{node}"
"puppet_enterprise::puppetdb_host": "puppetdb.rspec"
"puppet_enterprise::database_host": "puppetdb.rspec"
"puppet_enterprise::pcp_broker_host": "master.rspec"
"pe_install::upgrade_from": "2017.2.2"
"puppet_enterprise::packages::installing": true
}
            HIERA
          end

          around(:each) do |example|
            run_with_custom_pe_conf(node, pe_conf, example)
          end

          include_examples 'a base run on the console node'
          include_examples 'no repo config during install or upgrade'

          it do
            is_expected.to have_a_resource("Service[Shutdown pe-console-services #{package_refs_string}]")
              .that_is_a_dependency_for_downstream_resources(*package_refs)
          end
          it do
            is_expected.to have_a_resource("Service[Shutdown pe-nginx #{package_refs_string}]")
              .that_is_a_dependency_for_downstream_resources(*package_refs)
          end
        end

        context 'when repairing' do
          let(:node) { 'consolerepair.rspec' }
          let(:layout) { 'none' }
          let(:pe_conf) do
            <<-HIERA
{
"puppet_enterprise::certificate_authority_host": "master.rspec"
"puppet_enterprise::puppet_master_host": "master.rspec"
"puppet_enterprise::console_host": "#{node}"
"puppet_enterprise::puppetdb_host": "puppetdb.rspec"
"puppet_enterprise::database_host": "puppetdb.rspec"
"puppet_enterprise::mcollective_middleware_hosts": ["master.rspec"]
"puppet_enterprise::pcp_broker_host": "master.rspec"
}
            HIERA
          end

          around(:each) do |example|
            run_with_custom_pe_conf(node, pe_conf, example)
          end

          include_examples 'a base run on the console node'

          it do
            is_expected.to have_a_resource('Package[postgresql-client]')
              .that_requires_an_upstream('Class[Puppet_enterprise::Repo::Config]')
          end
          it do
            is_expected.to have_a_resource('Package[pe-puppet-enterprise-release]')
              .that_requires_an_upstream('Class[Puppet_enterprise::Repo::Config]')
          end
          it do
            is_expected.to have_a_resource('Package[pe-console-services]')
              .that_requires_an_upstream('Class[Puppet_enterprise::Repo::Config]')
          end

          it { is_expected.to_not have_a_resource("Service[Shutdown pe-console-services #{package_refs_string}]") }
          it { is_expected.to_not have_a_resource("Service[Shutdown pe-nginx #{package_refs_string}]") }
        end
      end
    end
  end
end
