require 'spec_helper'

RSpec.shared_examples 'a base run' do
  it { is_expected.to compile.with_all_deps }
  it "compiles without warnings" do
    is_expected.to compile
    expect(@logs.select { |l| l.level == :warning }.map { |l| l.message }).to eq([])
  end

  it { is_expected.to contain_class('puppet_enterprise::profile::certificate_authority') }
  it { is_expected.to contain_class('puppet_enterprise::profile::certificate_authority') }
  it { is_expected.to contain_class('puppet_enterprise::profile::certificate_authority') }
  it { is_expected.to contain_class('puppet_enterprise::profile::master') }
  it { is_expected.to contain_class('puppet_enterprise::profile::puppetdb') }
  it { is_expected.to contain_class('puppet_enterprise::profile::console') }
  it { is_expected.to contain_class('puppet_enterprise::profile::orchestrator') }
  it { is_expected.to contain_class('puppet_enterprise::profile::agent') }
  it { is_expected.to contain_class("pe_repo::platform::#{platform_class}") }
  it { is_expected.to contain_class('pe_repo::platform::windows_x86_64') }

  it do
    is_expected.to(have_a_resource('Package[pe-orchestration-services]')
                   .that_is_contained_by('Class[Puppet_enterprise::Packages]'))
  end

  it do
    is_expected.to(have_a_resource('Class[Puppet_enterprise::Packages]')
                   .that_is_a_dependency_for_a_downstream('Class[Puppet_enterprise::Profile::Orchestrator]'))
  end

  it do
    is_expected.to(have_a_resource('Class[Puppet_enterprise::Profile::Orchestrator]')
                   .that_requires_an_upstream('Class[Puppet_enterprise::Packages]'))
  end

  it do
    is_expected.to(have_a_resource('Class[Puppet_enterprise::Packages]')
                   .that_is_a_dependency_for_downstream_resources(
                     'Class[Puppet_enterprise::Profile::Certificate_authority]',
                     'Class[Puppet_enterprise::Profile::Master]',
                     'Class[Puppet_enterprise::Profile::Puppetdb]',
                     'Class[Puppet_enterprise::Profile::Console]',
                     'Class[Puppet_enterprise::Profile::Orchestrator]',
                     'Class[Puppet_enterprise::Profile::Agent]',
                   ))
  end

  it do
    is_expected.to have_a_resource('Package[pe-puppetdb]').that_is_contained_by('Class[Puppet_enterprise::Packages]')
    is_expected.to have_a_resource('Package[pe-puppetserver]').that_is_contained_by('Class[Puppet_enterprise::Packages]')
    is_expected.to have_a_resource('Package[pe-console-services]').that_is_contained_by('Class[Puppet_enterprise::Packages]')
    is_expected.to have_a_resource('Package[pe-orchestration-services]').that_is_contained_by('Class[Puppet_enterprise::Packages]')
  end

  it do
    is_expected.to(have_a_resource('File[/etc/puppetlabs/orchestration-services/conf.d/webserver.conf]')
                   .that_is_contained_by('Class[Puppet_enterprise::Profile::Orchestrator]'))
  end

  it do
    is_expected.to(have_a_resource('Puppet_enterprise::Trapperkeeper::Orchestrator[orchestration-services]')
        .that_is_contained_by('Class[Puppet_enterprise::Profile::Orchestrator]'))
  end

  it do
    is_expected.to(have_a_resource('Package[pe-r10k]')
      .that_requires_an_upstream('Package[pe-client-tools]'))
  end

  # The password string is shell escaped, so we must NOT quote it.
  it { is_expected.to contain_exec('set console admin password')
      .with_command(/puppet-infra console_password/) }
end

RSpec.shared_examples 'a base install' do
  include_context 'upgrade service shutdown package refs'

  it { is_expected.to_not have_a_resource("Service[Shutdown pe-orchestration-services #{package_refs_string}]") }
  it { is_expected.to_not have_a_resource("Service[Shutdown pe-puppetserver #{package_refs_string}]") }
  it { is_expected.to_not have_a_resource("Service[Shutdown pe-puppetdb #{package_refs_string}]") }
  it { is_expected.to_not have_a_resource("Service[Shutdown pe-postgresql #{package_refs_string}]") }
  it { is_expected.to_not have_a_resource("Service[Shutdown pe-console-services #{package_refs_string}]") }
  it { is_expected.to_not have_a_resource("Service[Shutdown pe-nginx #{package_refs_string}]") }
end

RSpec.shared_examples 'a base upgrade' do
  include_context 'upgrade service shutdown package refs'

  it do
    is_expected.to have_a_resource("Service\[Shutdown pe-orchestration-services #{package_refs_string}]")
      .that_is_a_dependency_for_downstream_resources(*package_refs)
  end
  it do
    is_expected.to have_a_resource("Service\[Shutdown pe-puppetserver #{package_refs_string}]")
      .that_is_a_dependency_for_downstream_resources(*package_refs)
  end
  it do
    is_expected.to have_a_resource("Service\[Shutdown pe-puppetdb #{package_refs_string}]")
      .that_is_a_dependency_for_downstream_resources(*package_refs)
  end
  it do
    is_expected.to have_a_resource("Service\[Shutdown pe-postgresql #{package_refs_string}]")
      .that_is_a_dependency_for_downstream_resources(*package_refs)
  end
  it do
    is_expected.to have_a_resource("Service\[Shutdown pe-console-services #{package_refs_string}]")
      .that_is_a_dependency_for_downstream_resources(*package_refs)
  end
  it do
    is_expected.to have_a_resource("Service\[Shutdown pe-nginx #{package_refs_string}]")
      .that_is_a_dependency_for_downstream_resources(*package_refs)
  end
end

describe 'monolithic host' do
  context 'supported operating systems' do
    on_supported_os.each do |os, base_facts|
      context "on #{os}", :os => os do
        include_context('hosts', os, base_facts)

        context 'with pe postgres' do
          let(:node) { 'master.rspec' }
          let(:layout) { 'mono' }

          include_examples 'a base run'
          include_examples 'a base install'
          include_examples 'no repo config during install or upgrade'

          it { is_expected.to contain_class('puppet_enterprise::profile::database') }
          it { is_expected.to contain_class('pe_postgresql::server') }
          it { is_expected.to contain_package('postgresql-server').that_comes_before('Class[puppet_enterprise::profile::database]') }
          it { is_expected.to have_a_resource('Package[postgresql-server]')
                .that_is_contained_by('Class[Pe_postgresql::Server]') }
          it { is_expected.to have_a_resource('Package[postgresql-client]') }
          it { is_expected.to have_a_resource('Package[postgresql-contrib]') }
          it { is_expected.to contain_package('pe-puppetdb').that_requires('Class[puppet_enterprise::profile::database]') }
          it { is_expected.to contain_package('pe-console-services').that_requires('Class[puppet_enterprise::profile::database]') }
          it { is_expected.to contain_package('pe-orchestration-services').that_requires('Class[puppet_enterprise::profile::database]') }

          it do
            is_expected.to have_a_resource('Exec[generate node cert]')
              .that_is_a_dependency_for_a_downstream('File[/opt/puppetlabs/server/data/postgresql/9.6/data/certs/_local.cert.pem]')
          end
        end

        context 'with external postgres' do
          let(:node) { 'master.rspec' }
          let(:layout) { 'mono-external-psql' }

          include_examples 'a base run'
          include_examples 'a base install'
          include_examples 'no repo config during install or upgrade'

          it { is_expected.to_not contain_class('puppet_enterprise::profile::database') }
          it { is_expected.to_not contain_class('pe_postgresql::server') }
          it { is_expected.to_not contain_package('postgresql-server') }
        end

        context 'upgrading' do
          # node is handled specially in spec/fixtures/manifests/site.pp to include
          # the pe_install::upgrade::postgres_9_6 class
          let(:node) { 'upgrademono.rspec' }
          let(:layout) { 'mono' }
          let(:context_facts) do
            {
              :pe_postgresql_info          => {
                'installed_server_version' => '9.4',
              }
            }
          end

          let(:pe_conf) do
            <<-HIERA
{
"puppet_enterprise::puppet_master_host": "#{node}"
}
            HIERA
          end

          around(:each) do |example|
            run_with_custom_pe_conf(node, pe_conf, example)
          end

          include_examples 'a base run'
          include_examples 'a base upgrade'
          include_examples 'no repo config during install or upgrade'

          it { is_expected.to compile.with_all_deps }

          it do
            is_expected.to have_a_resource('Exec[copy postgresql.conf from old install location to new install location]')
              .that_requires_an_upstream('Package[postgresql-server]')
              .that_is_a_dependency_for_downstream_resources(
                'Class[Pe_postgresql::Server::Config]',
                'Pe_install::Stop_service[Stop pe-postgresql before pg_upgrade]',
                'Service[Shutdown pe-postgresql before [Exec[migrate to new postgres]]]'
              )
          end

          it do
            is_expected.to(
              have_a_resource('Class[Pe_install::Upgrade::Postgres::Backup_binaries]')
                .that_is_a_dependency_for_downstream_resources(
                  'Package[postgresql-client]',
                  'Package[postgresql-server]',
                  'Package[postgresql-contrib]',
                  'Package[pe-postgresql96-pglogical]',
                  'Pe_install::Stop_service[Stop pe-postgresql before pg_upgrade]',
                  'Service[Shutdown pe-postgresql before [Exec[migrate to new postgres]]]'
                )
            )
          end

          it do
            is_expected.to(
              have_a_resource('Service[Shutdown pe-postgresql before [Exec[migrate to new postgres]]]')
                .that_is_a_dependency_for_a_downstream('Exec[migrate to new postgres]')
            )
          end

          it do
            is_expected.to(
              have_a_resource('Exec[migrate to new postgres]')
                .that_is_a_dependency_for_a_downstream('Class[Pe_postgresql::Server::Config]')
                .that_requires_an_upstream('Class[Pe_postgresql::Server::Initdb]')
            )
          end

          it do
            is_expected.to(
              have_a_resource('Class[puppet_enterprise::pg::pglogical::drop_orphaned_triggers]')
                .that_is_a_dependency_for_a_downstream('Exec[vacuum the newly migrated database]')
                .that_requires_an_upstream('Class[Pe_postgresql::Server::Service]')
            )
          end

          it do
            is_expected.to(
              have_a_resource('Pe_postgresql_psql[Drop pglogical trigger pglogical_truncate_trigger_add for pe-classifier sql]')
                .that_is_a_dependency_for_a_downstream('Exec[vacuum the newly migrated database]')
                .that_requires_an_upstream('Class[Pe_postgresql::Server::Service]')
            )
          end

          it do
            is_expected.to(
              have_a_resource('Exec[vacuum the newly migrated database]')
                .that_requires_an_upstream('Class[Pe_postgresql::Server::Service]')
            )
          end
        end

        context 'repairing' do
          let(:node) { 'repairmono.rspec' }
          let(:layout) { 'none' }

          let(:pe_conf) do
            <<-HIERA
{
"puppet_enterprise::puppet_master_host": "#{node}"
}
            HIERA
          end

          around(:each) do |example|
            run_with_custom_pe_conf(node, pe_conf, example)
          end

          include_examples 'a base run'

          it do
            is_expected.to have_a_resource('Class[Puppet_enterprise::Repo::Config]')
              .that_is_a_dependency_for_downstream_resources(
                'Package[postgresql-client]',
                'Package[postgresql-contrib]',
                'Package[postgresql-server]',
                'Package[pe-puppetdb]',
                'Package[pe-puppetserver]',
                'Package[pe-console-services]',
                'Package[pe-orchestration-services]'
              )
          end

          it do
            is_expected.to have_a_resource('Package[postgresql-client]')
              .that_requires_an_upstream('Package[pe-puppet-enterprise-release]')
          end

          context 'managed pe-postgres' do
            let(:node) { 'repairmonomanagedpostgres.rspec' }
            let(:layout) { 'none' }

            let(:pe_conf) do
              <<-HIERA
{
"puppet_enterprise::puppet_master_host": "#{node}"
"puppet_enterprise::database_host": "postgres.rspec"
}
              HIERA
            end

            include_examples 'a base run'

            it { is_expected.to_not contain_class('puppet_enterprise::profile::database') }
            it { is_expected.to_not contain_class('pe_postgresql::server') }
            it { is_expected.to_not contain_package('postgresql-server') }

            it do
              is_expected.to have_a_resource('Class[Puppet_enterprise::Repo::Config]')
                .that_is_a_dependency_for_downstream_resources(
                  'Package[postgresql-client]',
                  'Package[pe-puppetdb]',
                  'Package[pe-puppetserver]',
                  'Package[pe-console-services]',
                  'Package[pe-orchestration-services]'
                )
            end

            it do
              is_expected.to have_a_resource('Package[postgresql-client]')
                .that_requires_an_upstream('Package[pe-puppet-enterprise-release]')
            end

            it do
              is_expected.to have_a_resource('File[/opt/puppetlabs/server/bin/validate_postgresql_connection.sh]')
                .that_requires_an_upstream('Package[pe-puppet-enterprise-release]')
            end
          end
        end
      end
    end
  end
end
