require 'spec_helper'

RSpec.shared_examples 'a base run on the puppetdb node' do
  it { is_expected.to compile.with_all_deps }
  it { is_expected.to contain_class('puppet_enterprise::profile::puppetdb') }
  it { is_expected.to contain_class('puppet_enterprise::profile::database') }
  it { is_expected.to_not contain_class('puppet_enterprise::profile::master') }
  it { is_expected.to_not contain_class('puppet_enterprise::profile::console') }
  it { is_expected.to contain_class('puppet_enterprise::profile::agent') }
  it do
    is_expected.to(have_a_resource('Class[Puppet_enterprise::Packages]')
                   .that_is_a_dependency_for_downstream_resources(
                     'Class[Puppet_enterprise::Profile::Puppetdb]',
                     'Class[Puppet_enterprise::Profile::Agent]',
                   ))
  end
  it { is_expected.not_to have_a_resource('Exec[set console admin password]') }
end

describe 'puppetdb host' do
  context 'supported operating systems' do
    on_filtered_os(:redhat, :debian) do |os, base_facts|
      context "on #{os}", :os => os do
        include_context('hosts', os, base_facts)
        include_context 'upgrade service shutdown package refs'
        let(:console_refs) { [] }

        let(:node) { 'puppetdb.rspec' }
        let(:layout) { 'split' }

        include_examples 'a base run on the puppetdb node'
        include_examples 'no repo config during install or upgrade'

        it { is_expected.to_not have_a_resource("Service[Shutdown pe-postgresql #{package_refs_string}]") }
        it { is_expected.to_not have_a_resource("Service[Shutdown pe-puppetdb #{package_refs_string}]") }

        context 'when upgrading' do
          let(:node) { 'puppetdbupgrade.rspec' }
          let(:layout) { 'none' }
          let(:pe_conf) do
            <<-HIERA
{
"puppet_enterprise::certificate_authority_host": "master.rspec"
"puppet_enterprise::puppet_master_host": "master.rspec"
"puppet_enterprise::console_host": "console.rspec"
"puppet_enterprise::puppetdb_host": "#{node}"
"puppet_enterprise::database_host": "#{node}"
"puppet_enterprise::pcp_broker_host": "master.rspec"
"pe_install::upgrade_from": "2017.2.2"
"puppet_enterprise::packages::installing": true
}
            HIERA
          end
          let(:context_facts) do
            {
              :pe_postgresql_info          => {
                'installed_server_version' => '9.4',
              }
            }
          end

          around(:each) do |example|
            run_with_custom_pe_conf(node, pe_conf, example)
          end

          include_examples 'a base run on the puppetdb node'
          include_examples 'no repo config during install or upgrade'

          it do
            is_expected.to have_a_resource("Service\[Shutdown pe-postgresql #{package_refs_string}]")
              .that_is_a_dependency_for_downstream_resources(*package_refs)
          end
          it do
            is_expected.to have_a_resource("Service[Shutdown pe-puppetdb #{package_refs_string}]")
              .that_is_a_dependency_for_downstream_resources(*package_refs)
          end
        end

        context 'when repairing' do
          let(:node) { 'puppetdbrepair.rspec' }
          let(:layout) { 'none' }
          let(:pe_conf) do
            <<-HIERA
{
"puppet_enterprise::certificate_authority_host": "master.rspec"
"puppet_enterprise::puppet_master_host": "master.rspec"
"puppet_enterprise::console_host": "console.rspec"
"puppet_enterprise::puppetdb_host": "#{node}"
"puppet_enterprise::database_host": "#{node}"
"puppet_enterprise::mcollective_middleware_hosts": ["master.rspec"]
"puppet_enterprise::pcp_broker_host": "master.rspec"
}
            HIERA
          end

          around(:each) do |example|
            run_with_custom_pe_conf(node, pe_conf, example)
          end

          include_examples 'a base run on the puppetdb node'

          it { is_expected.to contain_package('pe-puppet-enterprise-release')
                .that_requires('Class[Puppet_enterprise::Repo]') }
          it { is_expected.to contain_package('pe-puppetdb')
                .that_requires('Class[Puppet_enterprise::Repo]') }
          it do
            is_expected.to have_a_resource('Class[Puppet_enterprise::Repo::Config]')
              .that_is_a_dependency_for_downstream_resources(
                'Package[postgresql-client]',
                'Package[postgresql-server]',
                'Package[postgresql-contrib]',
                'Package[pe-puppet-enterprise-release]',
                'Package[pe-puppetdb]',
              )
          end

          it do
            is_expected.to_not have_a_resource("Service\[Shutdown pe-postgresql #{package_refs_string}]")
          end
          it do
            is_expected.to_not have_a_resource("Service[Shutdown pe-puppetdb #{package_refs_string}]")
          end
        end
      end
    end
  end
end
