require 'spec_helper'

describe 'replica host' do
  context 'supported operating systems' do
    on_filtered_os(:redhat, :debian, :sles).each do |os, base_facts|
      context "on #{os}", :os => os do
        include_context('hosts', os, base_facts)

        let(:node) { 'replica.rspec' }
        let(:layout) { 'none' }

 	let(:pre_condition) do
          <<-MANIFEST
            function puppet_enterprise::is_using_mco()  { false }
            include puppet_enterprise
            include puppet_enterprise::profile::primary_master_replica
          MANIFEST
        end

        let(:pe_conf) do
          <<-HIERA
{
"puppet_enterprise::puppet_master_host": "master.rspec"
"puppet_enterprise::profile::master::code_manager_auto_configure": true
"puppet_enterprise::profile::master::provisioned_replicas": [
  "replica.rspec"
],
}
            HIERA
        end

        around(:each) do |example|
          run_with_custom_pe_conf(node, pe_conf, example)
        end

        it { is_expected.to compile.with_all_deps }

        it do
          is_expected.to contain_class('puppet_enterprise::profile::master')
            .with_ca_host('master.rspec')
            .with_master_of_masters_certname('replica.rspec')
            .with_replication_mode('replica')
        end

        it do
          is_expected.to contain_class('puppet_enterprise::profile::database')
            .with_replication_source_hostname('master.rspec')
            .with_replication_mode('replica')
        end

        it do
          is_expected.to contain_class('puppet_enterprise::profile::puppetdb')
            .with_database_host('replica.rspec')
            .with_master_certname('replica.rspec')
            .with_rbac_host('replica.rspec')
            .with_sync_peers([
              {
                'host' => 'master.rspec',
                'port' => '8081',
                'sync_interval_minutes' => '2',
              },
            ])
        end

        it do
          is_expected.to contain_class('puppet_enterprise::profile::console')
            .with_database_host('replica.rspec')
            .with_master_host('replica.rspec')
            .with_puppetdb_host(['replica.rspec'])
            .with_replication_mode('replica')
        end

        it do
          is_expected.to have_a_resource('Class[Puppet_enterprise::Repo::Config]')
            .that_is_a_dependency_for_downstream_resources(
              'Package[postgresql-client]',
              'Package[postgresql-server]',
              'Package[postgresql-contrib]',
              'Package[pe-puppetdb]',
              'Package[pe-puppetserver]',
              'Package[pe-console-services]',
              'Package[pe-orchestration-services]',
              'Package[pe-installer]',
            )
        end

        it do
          is_expected.to have_a_resource('Package[postgresql-client]')
            .that_requires_an_upstream('Package[pe-puppet-enterprise-release]')
        end

        it do
          is_expected.to have_a_resource('File[/opt/puppetlabs/server/bin/validate_postgresql_connection.sh]')
            .that_requires_an_upstream('Package[pe-puppet-enterprise-release]')
        end
      end
    end
  end
end
