RSpec.shared_context 'hosts', :hosts_context do |os, base_facts|
  let(:layout) { 'test' }
  let(:hiera_config) { 'spec/fixtures/hiera/hiera.yaml' }
  let(:platform_tag) { platform_tag_for(os, base_facts) }
  let(:aio_agent_build) { '1.5.0' }
  let(:fqdn) { node }
  let(:hostname) { node.sub(/\..*/,'') }
  let(:platform_class) { platform_tag.gsub('.','').gsub('-','_') }
  # Can override this in an example group to inject facts
  let(:context_facts) { {} }
  let(:facts) do
    base_facts.merge({
      :platform_tag    => platform_tag,
      :layout          => layout,
      :fqdn            => fqdn,
      :hostname        => hostname,
      :aio_agent_build => aio_agent_build,
    }).merge(context_facts)
  end

  before(:each) do
    local_aio_agent_build = aio_agent_build
    Puppet::Parser::Functions.newfunction(:pe_compiling_server_aio_build, :type => :rvalue) do |args|
      local_aio_agent_build
    end
    Puppet::Parser::Functions.newfunction(:pe_servername, :type => :rvalue) do |args|
      # servername is not set during puppet apply
      nil
    end
  end
end

RSpec.shared_context 'upgrade service shutdown package refs' do
  let(:pg_refs) do
    [
      'Package[postgresql-client]',
      'Package[postgresql-server]',
      'Package[postgresql-contrib]',
    ]
  end
  let(:console_refs) do
    [
      'Package[pe-nginx]',
    ]
  end
  let(:package_refs) do
    [
      'Class[Puppet_enterprise::Packages]',
    ] + pg_refs + console_refs
  end 
  let(:package_refs_string) { "before [#{package_refs.join(", ")}]" }
end

RSpec.shared_context 'SSL stubs' do
  before(:each) do
    # Set SSL lookup to memory for puppets network pool
    # copied from https://github.com/puppetlabs/puppet/blob/master/spec/unit/network/http_pool_spec.rb
    Puppet::SSL::Key.indirection.terminus_class = :memory
    Puppet::SSL::CertificateRequest.indirection.terminus_class = :memory

    # Stub for new method of certificate handling in Puppet 6
    # Adapted from https://github.com/puppetlabs/puppet/blob/master/spec/unit/network/http_pool_spec.rb
    ca_cert_file = File.expand_path('/path/to/ssl/certs/ca_cert.pem')
    Puppet[:ssl_client_ca_auth] = ca_cert_file
    allow(Puppet::FileSystem).to receive(:exist?).with(ca_cert_file).and_return(true)
    host_cert_file = File.expand_path('/path/to/ssl/certs/host_cert.pem')
    allow(Puppet::FileSystem).to receive(:exist?).with(host_cert_file).and_return(true)
    Puppet[:hostcert] = host_cert_file
    host = double("host", :certificate => double('cert',:content=>'real_cert'), :key => double('key',:content=>'real_key'), :ssl_store => double("store"))
    allow(Puppet::SSL::Host).to receive(:localhost).and_return(host)
  end
end

module PEInstall
module PlatformHelper

  def platform_tag_for(os, facts)
    case os
    when /redhat/i
      then "el-#{facts[:os]['release']['major']}-#{facts[:architecture]}"
    when /ubuntu/i
      then "ubuntu-#{facts[:os]['release']['major']}-#{facts[:architecture]}"
    when /sles/i
      then "sles-#{facts[:os]['release']['major']}-#{facts[:architecture]}"
    else
      raise("Don't know how to create a platform_tag fact for os '#{os}'")
    end
  end

end
end
