require 'rspec-puppet-facts'
require 'matchers/shared_matchers'
require 'shared/contexts'
require 'shared/examples'
require 'shared/rspec_puppet_facts'
require 'shared/custom_pe_conf'

module PEInstall
  SPEC_BASE_DIR = File.expand_path(File.dirname(__FILE__))
end

# Emulate $LOAD_PATH for puppet subcommands:
# https://github.com/puppetlabs/puppet/blob/4.5.3/lib/puppet/util/command_line.rb#L110-L119
# This is most important for being able to load hiera/backend/hocon_backend.rb from pe_manager
Dir.glob('spec/fixtures/modules/*').each do |f|
  next if ['pe_install','dependency_matchers'].include?(File.basename(f))
  $LOAD_PATH << "#{f}/lib" if File.directory?(f)
end

RSpec.configure do |config|
  # Re-override the mock_with :mocha that is in
  # puppetlabs_spec_helper-1.1.1/lib/puppetlabs_spec_helper/puppet_spec_helper.rb:122
  config.mock_with :rspec
end
require 'puppetlabs_spec_helper/module_spec_helper'

RSpec.configure do |config|
  config.extend(RspecPuppetFacts)
  config.extend(PEInstall::RspecPuppetFacts)
  config.include(PEInstall::DependencyMatchers)
  config.include(PEInstall::CustomPEConf)
  config.include(PEInstall::PlatformHelper)
  config.default_facts = {
    :pe_concat_basedir         => '/tmp/file',
    :platform_symlink_writable => true,
    # pe_build needs to be set to make `is_upgrade` work in params.pp
    :pe_build                  => '2016.4.4',
    :identity => {
      'privileged' => true,
    },
    :mountpoints => {
      '/tmp' => {
        'options' => ['rw'],
      }
    },
    :puppet_files_dir_present => true,
    :puppetversion => '4.10.4',
    :aio_agent_version => '1.10.4',
    :aio_agent_build   => '1.10.4',
    # Only needed for 2016.4.x use of ::memory fact which is not present in
    # facter 2.4.6, which is the latest version of facter available as a gem.
    :memory           => {
      'system'        => {
        'total_bytes' => 8589934592,
      }
    },
    :memorysize => '8 GiB',
    :osfamily => 'RedHat',
    :operatingsystem => 'CentOS',
    :processorcount => '2',
  }
  config.manifest = 'spec/fixtures/manifests/site.pp'
  config.after(:suite) do
    nodes_dir = File.expand_path("spec/fixtures/hiera/conf.d/nodes")
    FileUtils.rmdir(nodes_dir) if File.exist?(nodes_dir) && Dir.glob("#{nodes_dir}/*").empty?
  end

  # The behavior of du is different on OSX, and it's not one of our targets for
  # the module. So we'll skip any test involving du on OSX.
  if RUBY_PLATFORM.include?('darwin') # assumes Macintosh
    config.filter_run_excluding :broken_on_osx
  end
end
