module PuppetX
  module Util
    class OrchestratorError < StandardError
    end

    class OrchestratorJobFailedError < OrchestratorError
      attr_reader :message, :report_url

      def initialize(message, report_url)
        super(_("An error has occurred while running puppet.") + " " +
          _("See the report at %{report_url} for more details. (%{message})") % { report_url: report_url, message: message })
        @message = message
        @report_url = report_url
      end
    end

    class OrchestratorJobStoppedError < OrchestratorError
      attr_reader :job_id

      def initialize(job_id)
        super(_("An error has occurred while running puppet.") + " " +
          _("The orchestrator job %{job_id} was externally stopped.") % { job_id: job_id })
        @job_id = job_id
      end
    end

    class OrchestratorJobTimeoutError < OrchestratorError
      attr_reader :timeout, :job_id

      def initialize(timeout, job_id)
        super(_("An error has occurred while running puppet.") + " " +
          _("The orchestration service returned an error.") + " " +
          _("Verify that the service is running and has no relevant alerts using 'puppet infra status', then check the status of the job with 'puppet job show %{job_id}'.") % { job_id: job_id })
        @timeout = timeout
        @job_id = job_id
      end
    end

    class OrchestratorEmptyResponseError < OrchestratorError
      def initialize()
        super(_("An error has occurred while running puppet.") + " " +
          _("The orchestration service returned an empty response.") + " " +
          _("See the orchestration services log file for more details."))
      end
    end

    class OrchestratorResponseError < OrchestratorError
      attr_reader :status_code, :body

      def initialize(status_code, body)
        super(_("An error has occurred while running puppet.") + " " +
          _("The orchestration service returned an empty response.") + " " +
          _("See the orchestration services log file for more details.") + " " +
          "(" + _("status") + " %{status_code}: %{body}" % { status_code: status_code, body: body })
        @status_code = status_code
        @body = body
      end
    end

  end
end
