class pe_manager::reinitialize_replication (
  $target_database_name         = '',
  $classifier_database_name     = $puppet_enterprise::classifier_database_name,
  $rbac_database_name           = $puppet_enterprise::rbac_database_name,
  $activity_database_name       = $puppet_enterprise::activity_database_name,
  $orchestrator_database_name   = $puppet_enterprise::orchestrator_database_name,
  $inventory_database_name      = $puppet_enterprise::inventory_database_name,
) inherits puppet_enterprise {

  class { '::pe_postgresql::globals':
    user      => 'pe-postgres',
    group     => 'pe-postgres',
    psql_path => "${puppet_enterprise::server_bin_dir}/psql",
  }

  include pe_postgresql::params

  # stop all the services

  [ 'pe-console-services'
  , 'pe-orchestration-services'
  , 'puppet'
  ].map |$service_name| {
    service { $service_name:
      ensure => stopped,
    }
  }

  # reinitialize all the databases, or just the target database if specified

  $subscription_name = sprintf('s%.14s', sha1($trusted['certname']))

  if $target_database_name != '' {
    $databases = [ $target_database_name ]
  } else {
    $databases = [
      $classifier_database_name,
      $rbac_database_name,
      $activity_database_name,
      $orchestrator_database_name,
      $inventory_database_name
    ]
  }

  $databases.map |$database_name| {
    # remove pglogical subscriptions and then pglogical itself
    puppet_enterprise::pg::pglogical::subscription { "${database_name}/${subscription_name}":
      ensure            => absent,
      subscription_name => $subscription_name,
      database          => $database_name
    } ->

    puppet_enterprise::psql { "drop pglogical extension for ${database_name} sql":
      db      => $database_name,
      command => "drop extension pglogical cascade",
    } ->

    # kill potentially leftover connections
    puppet_enterprise::psql { "kill pglogical connections for ${database_name} sql":
      db      => 'pe-postgres',
      command => "SELECT pg_terminate_backend(pg_stat_activity.pid)
                         FROM pg_stat_activity
                         WHERE pg_stat_activity.datname = '${database_name}'"
    } ->

    # drop the database
    puppet_enterprise::psql { "drop ${database_name} sql":
      db => 'pe-postgres',
      command => "DROP DATABASE IF EXISTS \"${database_name}\""
    }
  }

}
