RSpec.shared_context 'puppet_context environmentpath' do
  let(:modulepath) { "#{fixtures_path}/modules" }
  let(:environmentpath) { "#{fixtures_path}/environments" } # fixture_path is defined in spec_helper.rb

  before(:each) do
    FileUtils.mkdir_p("#{environmentpath}/#{environment}")
    FileUtils.ln_s(modulepath, "#{environmentpath}/#{environment}/modules")
    FileUtils.touch("#{environmentpath}/#{environment}/environment.conf")
  end

  after(:each) do
    FileUtils.rm_rf("#{environmentpath}")
  end
end

RSpec.shared_context 'SSL stubs' do
  before(:each) do
    # Set SSL lookup to memory for puppets network pool
    # copied from https://github.com/puppetlabs/puppet/blob/master/spec/unit/network/http_pool_spec.rb
    Puppet::SSL::Key.indirection.terminus_class = :memory
    Puppet::SSL::CertificateRequest.indirection.terminus_class = :memory

    # Stub for new method of certificate handling in Puppet 6
    # Adapted from https://github.com/puppetlabs/puppet/blob/master/spec/unit/network/http_pool_spec.rb
    ca_cert_file = File.expand_path('/path/to/ssl/certs/ca_cert.pem')
    Puppet[:ssl_client_ca_auth] = ca_cert_file
    allow(Puppet::FileSystem).to receive(:exist?).with(ca_cert_file).and_return(true)
    host_cert_file = File.expand_path('/path/to/ssl/certs/host_cert.pem')
    allow(Puppet::FileSystem).to receive(:exist?).with(host_cert_file).and_return(true)
    Puppet[:hostcert] = host_cert_file
    host = double("host", :certificate => double('cert',:content=>'real_cert'), :key => double('key',:content=>'real_key'), :ssl_store => double("store"))
    allow(Puppet::SSL::Host).to receive(:localhost).and_return(host)
  end
end

RSpec.shared_context 'facter stubs' do
  # Only needed if WebMock.disable_net_connect! and running specs in a Platform9 VM
  before(:each) do 
    stub_request(:get, "http://169.254.169.254/latest/meta-data/").to_return(status: 200, body: "", headers: {})
    stub_request(:get, "http://169.254.169.254/latest/user-data/").to_return(status: 200, body: "", headers: {})
    stub_request(:get, "http://metadata/computeMetadata/v1beta1/?alt=json&recursive=true").to_return(status: 200, body: "", headers: {})
  end
end

RSpec.shared_examples 'help' do
  let(:face_help) { Puppet::Face[:help, :current].help('infrastructure', "#{action}") }
  let(:infra_help) { PuppetX::Util::InfrastructureHelp.action_help(:"#{action}") }

  before do
    allow($stdout).to receive(:puts)
  end

  it 'does not raise error' do
    expect { face_help }.to_not raise_error
    expect { infra_help }.to_not raise_error
    expect(face_help).to match (/USAGE/)
    expect(face_help).to match (/OPTIONS/)
  end

  it 'contains DESCRIPTION, USAGE and OPTIONS' do
    expect(infra_help).to_json match (/DESCRIPTION/)
    expect(infra_help).to_json match (/USAGE/)
    expect(infra_help).to_json match (/OPTIONS/)
  end
end
