RSpec.shared_context 'pe.conf recover context', :pe_conf_recover_context do |hiera_version|
  let(:master_node) { 'mono.rspec' }
  let(:console_node) { master_node }
  let(:puppetdb_node) { [master_node] }
  let(:database_node) { master_node }
  let(:scenario) { 'mono' }
  let(:using_nc) { true }
  let(:using_pe_repo_config_data) { false }
  let(:environment) { 'production' }
  let(:facts) { {} }
  let(:nodes) do
    [
      master_node,
      console_node,
      puppetdb_node,
      database_node,
    ].flatten.uniq
  end
  let(:base_pe_conf) do
    {
      'puppet_enterprise::puppet_master_host' => master_node,
      'puppet_enterprise::puppetdb_host'      => puppetdb_node,
      'puppet_enterprise::database_host'      => database_node,
      'puppet_enterprise::console_host'       => console_node,
    }
  end
  let(:nc_overrides) {
    { 'puppet_enterprise::send_analytics_data' => false }
  }

  # Some manifest defaults, only used if we conditionally_prepare_some_manifest_code
  let(:manifest_name) { 'site.pp' }
  # This relies on the tmp environmentpath created by Puppet's
  # lib/puppet/test/test_helper.rb
  let(:manifests_dir) { "#{Puppet[:environmentpath]}/#{environment}/manifests" }
  let(:manifest) { "#{manifests_dir}/#{manifest_name}" }

  def conditionally_prepare_some_manifest_code
    if respond_to?(:manifest_code)
      FileUtils.mkdir(manifests_dir) unless File.exist?(manifests_dir)
      File.write(manifest, manifest_code)
    end
  end

  after(:each) do
    FileUtils.rm(manifest) if respond_to?(:manifest_code) && File.exist?(manifest)
  end

  before(:each) do
    Puppet[:log_level] = 'debug'
    Puppet.settings[:hiera_config] = "spec/fixtures/hiera_#{hiera_version}/hiera.yaml"

    conditionally_prepare_some_manifest_code

    mocked_engine = Puppet::Util::Pe_conf::Recover.new
    expect(mocked_engine).to receive(:using_node_classifier?).
      and_return(using_nc)
    expect(mocked_engine).to receive(:get_node_terminus).
        and_return(nil)
    expect(mocked_engine).to receive(:get_core_infra_nodes).
      with(environment).and_return(nodes)
    expect(mocked_engine).to receive(:pe_conf_from_puppetdb).
      with(environment).and_return(base_pe_conf)

    nodes.each do |node|
      node_facts = facts.merge({
        'scenario' => scenario,
      })
      if using_nc
        expect(mocked_engine).to receive(:classifier_overrides_for_node).
          with(node, node_facts, nil).at_least(:once).and_return(nc_overrides)
      else
        expect(mocked_engine).not_to receive(:classifier_overrides_for_node)
      end

      expect(mocked_engine).to receive(:facts_for_node).
        with(node, 'production').and_return(node_facts)
      expect(mocked_engine).to receive(:resources_applied_to_node).
        with(node, 'Puppet_enterprise', environment).and_return(
          {
            'puppet_enterprise::master::puppetserver::jruby_max_active_instances' => 10001,
            'puppet_enterprise::manage_symlinks' => false
          })
      if using_pe_repo_config_data && node == 'mono.rspec'
        expect(mocked_engine).to receive(:resources_applied_to_node).
          with(node, 'Pe_repo', environment).and_return({'pe_repo::compile_master_pool_address' => 'test_server'})
        expect(mocked_engine).to receive(:find_hiera_overrides).
          with(node, ['pe_repo::compile_master_pool_address'],  {"scenario"=>"mono"}, environment, nil).and_return({'pe_repo::compile_master_pool_address' => 'test_server'})
        expect(mocked_engine).to receive(:find_hiera_overrides).
          with(node, ["puppet_enterprise::master::puppetserver::jruby_max_active_instances", "puppet_enterprise::manage_symlinks"],  {"scenario"=>"mono"}, environment, nil).and_return({'pe_repo::compile_master_pool_address' => 'test_server'})
        expect(mocked_engine).to receive(:find_hiera_overrides).
          with(node, ["puppet_enterprise_classifier_data_backend_present"],  {"scenario"=>"mono"}, environment, nil).and_return({'puppet_enterprise_classifier_data_backend_present' => true})
      else
        expect(mocked_engine).to receive(:resources_applied_to_node).
          with(node, 'Pe_repo', environment).and_return({})
      end
    end

    subject.recovery_engine = mocked_engine
    # For specs, bypass using the default environmentpath which is unlikely to
    # be set up, and instead pass in the dummy but functional environmentpath
    # created by rspec-puppet.
    subject.recover(:pe_environmentpath => Puppet[:environmentpath])
  end
end
