require 'spec_helper'
require 'puppet/face'
require 'webmock/rspec'

describe Puppet::Face[:infrastructure, :current] do
  let(:action) { subject.get_action(:console_password) }

  context "console_password action" do
    before(:each) do
      WebMock.disable_net_connect!
      allow(PuppetX::Util::PasswordUtils).to receive(:get_user).and_return("token")
      allow(PuppetX::Util::PasswordUtils).to receive(:reset_password).and_return(Net::HTTPResponse.new(nil, "200", "body"))
    end
    
    after(:each) do
      WebMock.allow_net_connect!
    end

    context "running as non-root" do
      it "tells the user to run the command as root" do
        allow(subject).to receive(:`).and_return("1000")
        expect(Puppet).to receive(:err).with("Please run this command as root")
        expect {subject.console_password}.to raise_error(SystemExit)
      end
    end

    context "running as root" do
      before(:each) do
        allow(subject).to receive(:`).and_return("0")
      end
      context "no options" do
        it "prompts the user for the password and confirmation matches" do
          allow(PuppetX::Util::Interview).to receive(:ask_password).with("Please enter the new console admin password").and_return("password")
          allow(PuppetX::Util::Interview).to receive(:ask_password).with("Please confirm the new console admin password").and_return("password")
          expect {subject.console_password}.to output("Password successfully changed\n").to_stdout
        end

        it "prompts the user for the password and confirmation differs" do
          allow(PuppetX::Util::Interview).to receive(:ask_password).with("Please enter the new console admin password").and_return("password")
          allow(PuppetX::Util::Interview).to receive(:ask_password).with("Please confirm the new console admin password").and_return("p@$$w0rd")
          expect(Puppet).to receive(:err).with("Passwords do not match. Password not changed.")
          expect {subject.console_password}.to raise_error(SystemExit)
        end
      end

      context "--password" do
        it "takes a password on the command line" do
          expect {subject.console_password({:password => 'password'})}.to output("Password successfully changed\n").to_stdout
        end

        it "asks the user to enter a password if password given is blank" do
          allow(PuppetX::Util::Interview).to receive(:ask_password).with("Please enter the new console admin password").and_return("password")
          allow(PuppetX::Util::Interview).to receive(:ask_password).with("Please confirm the new console admin password").and_return("password")
          expect {subject.console_password({:password => ""})}.to output("Password successfully changed\n").to_stdout
        end
      end
    end
  end
end
