require 'spec_helper'
require 'puppet/face'
require 'puppet/network/http_pool'
require 'puppet_x/util/infrastructure_error'
require 'hocon'
require 'webmock/rspec'


describe Puppet::Face[:infrastructure, :current] do
  let(:action) { subject.get_action(:provision) }

  context "provision action" do

    context 'with help example flag' do
      let(:action) { 'provision' }

      include_examples 'help'
    end

    context 'flag' do
      context '--master-certname' do

        it 'uses the master certname provided instead of fqdn' do
          # doesn't work without stubbing services config
          # result = infrastructure.provision('replica',
          #                                   'replica.net',
          #                                   {:master_certname => 'mom.net'})
          # expect(result).to be(1)
        end
      end

      context '--token-file' do

        it 'uses the token at provided location' do
          # current error: can't modify frozen string; provision.rb:51:in `strip!'
          # result = infrastructure.provision('replica',
          #                                   'replica.net',
          #                                   {:token_file => '/not/a/real/path'})
          # expect(result).to be(1)
        end
      end

      context 'provisioning invocation' do
        context 'when HA-only state has not been set up' do
          it 'submits the correct node groups'
          it 'changes classification correctly'
          it 'messages that classification was updated'
          it 'runs puppet'
          it 'messages provisioning finished'
        end
        context 'when HA-only state has been set up' do
          it 'changes classification correctly'
          it 'messages that classification was updated'
          it 'runs puppet'
          it 'messages provisioning finished'
        end
        context 'when the replica has already been provisioned' do
          it 'does something smart'
        end
      end

      context 'error cases: ' do
        context 'Unknown role' do
          it 'rasies an error' do
            expect { subject.provision('not-a-role', 'replica.net', {:token_file => '/not/a/real/path'}) }.to raise_error(PuppetX::Util::InfrastructureUnknownRoleError)
          end
        end
        context 'Cannot load services config' do
          it 'rasies an error'
        end
        context 'Not all required services running' do
          it 'rasies an error'
        end
        context 'Cannot query or write to NC' do
          it 'rasies an error'
        end
        context 'Cannot run Puppet' do
          it 'rasies an error'
        end
        context 'Cannot kick File Sync' do
          it 'rasies an error'
        end
      end
    end
  end
end
