require 'spec_helper'
require 'puppet_x/puppetlabs/meep/configure/psql'

describe 'PuppetX::Puppetlabs::Meep::Configure::PSQL' do
  let(:args) { {} }
  let(:psql) { PuppetX::Puppetlabs::Meep::Configure::PSQL.new(args) }
  let(:logs) { [] }

  around(:each) do |test|
    Puppet::Util::Log.with_destination(Puppet::Test::LogCollector.new(logs)) do
      test.run
    end
  end

  context 'local pe-postgres access' do
    it { expect(psql.command('foo')).to eq("/opt/puppetlabs/server/bin/psql -q -P tuples_only=on -X --command='foo'") }

    it "processes encoding data" do
      expect(Puppet::Util::Execution).to receive(:execute).with(match(/server_encoding/), anything).and_return('a')
      expect(Puppet::Util::Execution).to receive(:execute).with(match(/lc_ctype/), anything).and_return('b')
      expect(Puppet::Util::Execution).to receive(:execute).with(match(/lc_collate/), anything).and_return('c')

      expect(psql.encoding_data).to eq(
        'server_encoding' => 'a',
        'lc_ctype' => 'b',
        'lc_collate' => 'c',
      )
    end
  end

  context 'user/password access' do
    let(:args) do
      {
        :user => 'user',
        :password => 'password',
        :database => 'database',
        :host => 'host',
        :port => 1234,
      }
    end

    it do
      expect(psql.command('foo')).to eq(
        %q{/opt/puppetlabs/server/bin/psql -q -P tuples_only=on -X --username='user' --host='host' --port='1234' --dbname='database' --command='foo'}
      )
    end

    it "grabs the server version" do
      expect(Puppet::Util::Execution).to receive(:execute).with(match(/select version/), include({
          :custom_environment => a_hash_including({
            'PGPASSWORD' => args[:password],
          })})).and_return('PostgreSQL 9.6.3 on x86_64-pc-linux-gnu, compiled by gcc (GCC) 4.4.7 20120313 (Red Hat 4.4.7-4), 64-bit')

      expect(psql.server_version).to eq('9.6.3')
    end
  end

  context 'cert access' do
    let(:args) do
      {
        :user => 'user',
        :database => 'database',
        :host => 'host',
        :port => 1234,
        :sslcert => '/etc/cert.pem',
        :sslkey => '/etc/key.pem',
        :sslmode => 'verify-full',
        :sslcacert => '/etc/ca.pem',
      }
    end

    it do
      expect(psql.command('foo')).to eq(
        %q{/opt/puppetlabs/server/bin/psql -q -P tuples_only=on -X --username='user' --host='host' --port='1234' --dbname='database' --command='foo'}
      )
    end

    it "grabs the server version" do
      expect(Puppet::Util::Execution).to receive(:execute).with(match(/select version/), include({
          :custom_environment => a_hash_including({
            'PGSSLCERT' => args[:sslcert],
            'PGSSLKEY' => args[:sslkey],
            'PGSSLMODE' => args[:sslmode],
            'PGSSLROOTCERT' => args[:sslcacert]
          })
        })).and_return('PostgreSQL 9.6.3 on x86_64-pc-linux-gnu, compiled by gcc (GCC) 4.4.7 20120313 (Red Hat 4.4.7-4), 64-bit')

      expect(psql.server_version).to eq('9.6.3')
    end
  end

  context 'test accessors' do
    let(:args) do
      {
        :user => 'user',
        :password => 'password',
        :database => 'database',
        :host => 'host',
        :port => 1234,
        :sslcert => '/etc/cert.pem',
        :sslkey => '/etc/key.pem',
        :sslmode => 'verify-full',
        :sslcacert => '/etc/ca.pem',
      }
    end

    it do expect(psql.user).to eq(args[:user]) end
    it do expect(psql.password).to eq(args[:password]) end
    it do expect(psql.database).to eq(args[:database]) end
    it do expect(psql.host).to eq(args[:host]) end
    it do expect(psql.port).to eq(args[:port]) end
    it do expect(psql.sslcert).to eq(args[:sslcert]) end
    it do expect(psql.sslkey).to eq(args[:sslkey]) end
    it do expect(psql.sslmode).to eq(args[:sslmode]) end
    it do expect(psql.sslcacert).to eq(args[:sslcacert]) end
  end
end
