require 'spec_helper'
require 'puppet_x/puppetlabs/meep/puppet_context'

describe "PuppetX::Puppetlabs::Meep::PuppetContext" do
  class TestClass
    include PuppetX::Puppetlabs::Meep::PuppetContext
  end

  let(:tester) { TestClass.new }
  let(:environment) { 'puppetcontext' }

  include_context('puppet_context environmentpath')

  it 'resets environment' do
    orig_env = Puppet.lookup(:current_environment)
    orig_env_setting = Puppet[:environment]
    expect(orig_env.name).to_not eq(environment.to_sym)
    expect(orig_env_setting).to_not eq(environment)

    tester.puppet_environment_context(environmentpath, environment) do
      env = Puppet.lookup(:current_environment)
      expect(env.name).to eq(environment.to_sym)
      expect(env.full_modulepath).to eq(["#{environmentpath}/#{environment}/modules"])
      expect(Puppet[:environment]).to eq(environment)
    end

    expect(Puppet.lookup(:current_environment)).to eq(orig_env)
    expect(Puppet[:environment]).to eq(orig_env_setting)
  end

  it 'resets environmentpath' do
    orig_envpath = Puppet[:environmentpath]
    expect(orig_envpath).to_not eq(environmentpath)

    tester.puppet_environment_context(environmentpath, environment) do
      expect(Puppet[:environmentpath]).to eq(environmentpath)
    end

    expect(Puppet[:environmentpath]).to eq(orig_envpath)
  end

  it 'resets settings' do
    test_hiera_yaml = '/foo/bar/hiera.yaml'
    orig_hiera_yaml = Puppet[:hiera_config]
    expect(orig_hiera_yaml).to_not eq(test_hiera_yaml)

    tester.puppet_environment_context(environmentpath, environment, '', :hiera_config => test_hiera_yaml) do
      expect(Puppet[:hiera_config]).to eq(test_hiera_yaml)
    end

    expect(Puppet[:hiera_config]).to eq(orig_hiera_yaml)
  end

  it 'resets after an exception' do
    test_hiera_yaml = '/foo/bar/hiera.yaml'
    orig_hiera_yaml = Puppet[:hiera_config]
    expect(orig_hiera_yaml).to_not eq(test_hiera_yaml)

    expect do
      tester.puppet_environment_context(environmentpath, environment, '', :hiera_config => test_hiera_yaml) do
        expect(Puppet[:hiera_config]).to eq(test_hiera_yaml)
        raise('boom')
      end
    end.to raise_exception('boom')

    expect(Puppet[:hiera_config]).to eq(orig_hiera_yaml)
  end

  it 'skips basemodulepath by default' do
    orig_base = Puppet[:basemodulepath]
    expect(orig_base).to_not eq('')

    tester.puppet_environment_context(environmentpath, environment) do
      expect(Puppet[:basemodulepath]).to eq('')
      env = Puppet.lookup(:current_environment)
      # single path
      expect(env.full_modulepath).to eq(["#{environmentpath}/#{environment}/modules"])
    end

    expect(Puppet[:basemodulepath]).to eq(orig_base)
  end

  it 'uses basemodulepath if present' do
    new_base = '/some/path:/some/other/path'
    orig_base = Puppet[:basemodulepath]
    expect(orig_base).to_not eq('')

    tester.puppet_environment_context(environmentpath, environment, new_base) do
      expect(Puppet[:basemodulepath]).to eq(new_base)
      env = Puppet.lookup(:current_environment)
      expect(env.full_modulepath).to eq([
        "#{environmentpath}/#{environment}/modules",
        '/some/path',
        '/some/other/path',
      ])
    end

    expect(Puppet[:basemodulepath]).to eq(orig_base)
  end
end
