require 'spec_helper'
require 'puppet/face'
require 'puppet/network/http_pool'
require 'hocon'
require 'webmock/rspec'
require 'puppet_x/util/ca'

describe 'PuppetX::Util::CA' do
  describe 'advance_ca_serial' do

    def call_with_temp_file(content, amount_to_advance)
      tempfile = Tempfile.new('ca_serial_test_')
      begin
        tempfile.write(content)
        tempfile.close
        PuppetX::Util::CA.advance_ca_serial(tempfile.path, amount_to_advance)
        IO.read(tempfile)
      ensure
        tempfile.close
        tempfile.unlink
      end
    end


    it 'increments the serial' do
      expect(call_with_temp_file("0042", 0x3e8)).to eq("042A")
    end

    it 'pads to 4 digits' do
      expect(call_with_temp_file("0042", 1)).to eq("0043")
    end

    it 'extends to 5 digits' do
      expect(call_with_temp_file("0042", 1000000)).to eq("F4282")
    end

    it 'reads the serial in hex' do
      expect(call_with_temp_file("0x00F", 0x3e8)).to eq("03F7")
    end
  end
end
