require 'puppet_x/util/interview'
require 'puppet_x/util/infrastructure_error'

describe PuppetX::Util::Interview do
  context "confirm" do

    it "it returns default" do
      stream = StringIO.new(" \n")
      expect(described_class.confirm("question", true, stream)).to eq(true)
    end

    it "Yes is true" do
      stream = StringIO.new("Yes\n")
      expect(described_class.confirm("question", false, stream)).to eq(true)
    end

    it "Y is true" do
      stream = StringIO.new("Y\n")
      expect(described_class.confirm("question", false, stream)).to eq(true)
    end

    it "n is false" do
      stream = StringIO.new("n\n")
      expect(described_class.confirm("question", false, stream)).to eq(false)
    end

    it "NO is false" do
      stream = StringIO.new("NO\n")
      expect(described_class.confirm("question", false, stream)).to eq(false)
    end

    it "tries to get useful data" do
      stream = StringIO.new("wrong\nno\n")
      expect(described_class.confirm("question", false, stream)).to eq(false)
    end

    it "Exits after three tries" do
      stream = StringIO.new("wrong\nleave me alone\nstop bothering me\nno")
      expect{ described_class.confirm("question", false, stream) }.to raise_error(SystemExit)
    end
  end

  context "ask_server_list" do
    it 'returns the server list' do
      stream = StringIO.new("example.com:8181,example.com:4443")
      expect(described_class.ask_server_list("servers?", stream)).to eq(['example.com:8181','example.com:4443'])
    end

    it 'Exits after 3 tries without parseable answer' do
      stream = StringIO.new("bad\nserver\nnames")
      expect{ described_class.ask_server_list("question", stream) }.to raise_error(SystemExit)
    end

    it "Exits after 3 tries with empty answers" do
      stream = StringIO.new("\n\n\n")
      expect{ described_class.ask_server_list("question", stream) }.to raise_error(SystemExit)
    end
  end

  context 'ask_topo' do
    it 'returns a valid option' do
      expect(described_class.get_topo({topology: 'mono'})).to eq (:a)
    end
    it 'errors with an invalid option' do
      expect { described_class.get_topo({topology: 'invalid'}) }.to raise_error(PuppetX::Util::InfrastructureUnknownTopologyError)
    end
  end
end
