require 'spec_helper'
require 'puppet_x/util/rbac'

describe PuppetX::Util::RBAC do
  let(:home) { File.expand_path('~') }

  context "default token path" do
    it "returns the default token path for the system" do
      result = described_class.default_token_path()
      expect(result).to eq("#{home}/.puppetlabs/token")
    end
  end

  context "load_token" do
    context "when not given a path" do
      it "loads the token from the default path" do
        expect(File).to receive(:file?).with("#{home}/.puppetlabs/token").and_return(true)
        allow(File).to receive(:read).and_return("")
        described_class.load_token
      end
    end

    context "when given a path" do
      it "loads the token from that path" do
        expect(File).to receive(:file?).with("#{home}/foo").and_return(true)
        allow(File).to receive(:read).and_return("")
        described_class.load_token("#{home}/foo")
      end
    end

    context "when the file does not exist" do
      it "raises an error" do
        expect(File).to receive(:file?).and_return(false)
        expect { described_class.load_token }.to raise_error(RuntimeError, /puppet access login/)
      end
    end

    it "strips whitespace from the read result" do
      tempfile = Tempfile.new('test_token')
      content = "\nTOKEN\n"
      tempfile.write(content)
      tempfile.close
      expect(described_class.load_token(tempfile.path)).to eq("TOKEN")
    end
  end
end
