require 'open3'

RSpec::Matchers.define :exit_with do |expected|
  actual = nil
  status = nil
  output = nil

  match do |command|
    output, status = Open3.capture2e(command)
    actual = status.exitstatus
    expected == actual
  end

  supports_block_expectations

  failure_message do |block|
    message = []
    message << "expected exit with code #{expected} but " +
      (actual.nil? ? " exit was not called" : "we exited with #{actual} instead")
    message << "\nOutput of command:\n#{output}"
    message.join("\n")
  end

  failure_message_when_negated do |block|
    message = []
    message << "expected that exit would not be called with #{expected}"
    message << "\nOutput of command:\n#{output}"
    message.join("\n")
  end

  description do
    "expect exit with #{expected}"
  end
end
