RSpec.configure do |rspec|
  # This config option will be enabled by default on RSpec 4,
  # but for reasons of backwards compatibility, you have to
  # set it on RSpec 3.
  #
  # It causes the host group and examples to inherit metadata
  # from the shared context.
  rspec.shared_context_metadata_behavior = :apply_to_host_groups
end

RSpec.shared_context "pe_build_version function", :shared_context => :metadata do
  before(:example) do
    mock_pe_build_version = double(:call => pe_build_version)
    Puppet::Parser::Functions.newfunction(:pe_build_version, :type => :rvalue) do |args|
      mock_pe_build_version.call()
    end
  end
end

RSpec.shared_examples 'pe_repo::el/deb/sles' do
  let(:params) do
    {
      :agent_version => aio_agent_build,
      :installer_build => platform,
      :pe_version => pe_build_version,
    }
  end

  it 'compiles' do
    is_expected.to compile.with_all_deps
  end

  it 'includes pe_repo with the agent version without sha' do
    is_expected.to contain_pe_repo__repo("#{platform} #{pe_build_version}")
      .with_agent_version(aio_agent_version)
  end

end

RSpec.configure do |rspec|
  rspec.include_context "pe_build_version function", :include_shared => true
end
