require 'spec_helper'
describe 'pe_staging::file', :type => :define do

  # forcing a more sane caller_module_name to match real usage.
  let(:facts) { { :caller_module_name=> 'spec' } }
  let(:params) { { :subdir => facts[:caller_module_name], } }

  describe 'when deploying via puppet' do
    let(:title) { 'sample.tar.gz' }
    let(:params) { { :source => 'puppet:///modules/staging/sample.tar.gz',
                     :subdir => facts[:caller_module_name], } }

    it {
      should contain_file('/opt/puppetlabs/server/data/staging')
      should contain_file('/opt/puppetlabs/server/data/staging/spec/sample.tar.gz')
      should_not contain_exec('/opt/puppetlabs/server/data/staging/spec/sample.tar.gz')
    }
  end

  describe 'when deploying via local' do
    let(:title) { 'sample.tar.gz' }
    let(:params) { { :source => '/nfs/sample.tar.gz',
      :target => '/usr/local/sample.tar.gz',
    } }

    it {
      should contain_file('/opt/puppetlabs/server/data/staging')
      should contain_file('/usr/local/sample.tar.gz')
      should_not contain_exec('/opt/puppetlabs/server/data/staging/spec/sample.tar.gz')
    }
  end

  describe 'when deploying via http' do
    let(:title) { 'sample.tar.gz' }
    let(:params) { { :source => 'http://webserver/sample.tar.gz',
                     :subdir => facts[:caller_module_name], } }

    it {
      should contain_file('/opt/puppetlabs/server/data/staging')
      should contain_exec('/opt/puppetlabs/server/data/staging/spec/sample.tar.gz').with( {
        :command => 'curl  -f -L -o sample.tar.gz http://webserver/sample.tar.gz',
        :path        => '/opt/puppetlabs/puppet/bin:/usr/local/bin:/usr/bin:/bin',
        :environment => nil,
        :cwd         => '/opt/puppetlabs/server/data/staging/spec',
        :creates     => '/opt/puppetlabs/server/data/staging/spec/sample.tar.gz',
        :logoutput   => 'on_failure',
      })
    }
  end

  describe 'when deploying via http with custom curl options' do
    let(:title) { 'sample.tar.gz' }
    let(:params) { {
      :source => 'http://webserver/sample.tar.gz',
      :curl_option => '-b',
      :subdir => facts[:caller_module_name],
    } }

    it {
      should contain_file('/opt/puppetlabs/server/data/staging')
      should contain_exec('/opt/puppetlabs/server/data/staging/spec/sample.tar.gz').with( {
        :command => 'curl -b -f -L -o sample.tar.gz http://webserver/sample.tar.gz',
        :path        => '/opt/puppetlabs/puppet/bin:/usr/local/bin:/usr/bin:/bin',
        :environment => nil,
        :cwd         => '/opt/puppetlabs/server/data/staging/spec',
        :creates     => '/opt/puppetlabs/server/data/staging/spec/sample.tar.gz',
        :logoutput   => 'on_failure',
      })
    }
  end

  describe 'when deploying via http with parameters' do
    let(:title) { 'sample.tar.gz' }
    let(:params) { { :source => 'http://webserver/sample.tar.gz',
      :target => '/usr/local/sample.tar.gz',
    } }

    it { should contain_file('/opt/puppetlabs/server/data/staging')
      should contain_exec('/usr/local/sample.tar.gz').with( {
        :command => 'curl  -f -L -o sample.tar.gz http://webserver/sample.tar.gz',
        :path        => '/opt/puppetlabs/puppet/bin:/usr/local/bin:/usr/bin:/bin',
        :environment => nil,
        :cwd         => '/usr/local',
        :creates     => '/usr/local/sample.tar.gz',
      })
    }
  end

  describe 'when deploying via https' do
     let(:title) { 'sample.tar.gz' }
     let(:params) { { :source => 'https://webserver/sample.tar.gz',
                      :subdir => facts[:caller_module_name], } }

     it { should contain_file('/opt/puppetlabs/server/data/staging') }
     it { should contain_exec('/opt/puppetlabs/server/data/staging/spec/sample.tar.gz').with( {
       :command => 'curl  -f -L -o sample.tar.gz https://webserver/sample.tar.gz',
       :path        => '/opt/puppetlabs/puppet/bin:/usr/local/bin:/usr/bin:/bin',
       :environment => nil,
       :cwd         => '/opt/puppetlabs/server/data/staging/spec',
       :creates     => '/opt/puppetlabs/server/data/staging/spec/sample.tar.gz',
       :logoutput   => 'on_failure',
     }) }
  end

  describe 'when deploying via https with parameters' do
    let(:title) { 'sample.tar.gz' }
    let(:params) { { :source => 'https://webserver/sample.tar.gz',
      :username => 'puppet',
      :password => 'puppet',
      :subdir => facts[:caller_module_name],
    } }

    it {
      should contain_file('/opt/puppetlabs/server/data/staging')
      should contain_exec('/opt/puppetlabs/server/data/staging/spec/sample.tar.gz').with( {
        :command => 'curl  -f -L -o sample.tar.gz -u puppet:puppet https://webserver/sample.tar.gz',
        :path        => '/opt/puppetlabs/puppet/bin:/usr/local/bin:/usr/bin:/bin',
        :environment => nil,
        :cwd         => '/opt/puppetlabs/server/data/staging/spec',
        :creates     => '/opt/puppetlabs/server/data/staging/spec/sample.tar.gz',
        :logoutput   => 'on_failure',
      })
    }
  end

  describe 'when deploying via ftp' do
    let(:title) { 'sample.tar.gz' }
    let(:params) { { :source => 'ftp://webserver/sample.tar.gz',
                     :subdir => facts[:caller_module_name], } }

    it {
      should contain_file('/opt/puppetlabs/server/data/staging')
      should contain_exec('/opt/puppetlabs/server/data/staging/spec/sample.tar.gz').with( {
        :command => 'curl  -o sample.tar.gz ftp://webserver/sample.tar.gz',
        :path        => '/opt/puppetlabs/puppet/bin:/usr/local/bin:/usr/bin:/bin',
        :environment => nil,
        :cwd         => '/opt/puppetlabs/server/data/staging/spec',
        :creates     => '/opt/puppetlabs/server/data/staging/spec/sample.tar.gz',
        :logoutput   => 'on_failure',
      })
    }
  end

  describe 'when deploying via ftp with parameters' do
    let(:title) { 'sample.tar.gz' }
    let(:params) { { :source => 'ftp://webserver/sample.tar.gz',
      :username => 'puppet',
      :password => 'puppet',
      :subdir => facts[:caller_module_name],
    } }

    it {
      should contain_file('/opt/puppetlabs/server/data/staging')
      should contain_exec('/opt/puppetlabs/server/data/staging/spec/sample.tar.gz').with( {
        :command => 'curl  -o sample.tar.gz -u puppet:puppet ftp://webserver/sample.tar.gz',
        :path        => '/opt/puppetlabs/puppet/bin:/usr/local/bin:/usr/bin:/bin',
        :environment => nil,
        :cwd         => '/opt/puppetlabs/server/data/staging/spec',
        :creates     => '/opt/puppetlabs/server/data/staging/spec/sample.tar.gz',
        :logoutput   => 'on_failure',
      })
    }
  end

end
