require 'spec_helper'

describe 'puppet_enterprise::cli_config' do
  require 'puppet/resource/catalog'
  require 'puppet/indirector/memory'

  class ::Puppet::Resource::Catalog::StoreConfigsTesting < Puppet::Indirector::Memory; end

  before(:each) do
    Puppet[:storeconfigs] = true
    Puppet.settings[:storeconfigs_backend] = 'store_configs_testing'

    Puppet::Parser::Functions.newfunction(:puppetdb_query,
                                          :type => :rvalue,
                                          :arity => 1) do |args|
      [{'certname' => 'master.dev',
        'tags' => [],
        'title' => 'Puppet_enterprise::Master::Code_manager',
        'parameters' => {'webserver_ssl_port' => 1234,
                         'puppet_master_port' => 4321}},
       {'certname' => 'replica.dev',
        'tags' => ['puppet_enterprise::profile::primary_master_replica'],
        'title' => 'Puppet_enterprise::Master::Code_manager',
        'parameters' => {'webserver_ssl_port' => 1234,
                         'puppet_master_port' => 4321}},
       {'certname' => 'replica.dev',
        'tags' => ['puppet_enterprise::profile::primary_master_replica'],
        'title' => 'Puppet_enterprise::Master::File_sync',
        'parameters' => {'puppetserver_webserver_ssl_port' => 4321,
                         'compile_master' => false,
                         'replication_mode' => 'replica',
                         'storage_service_disabled' => true,}},
       {'certname' => 'master.dev',
        'tags' => [],
        'title' => 'Puppet_enterprise::Profile::Puppetdb',
        'parameters' => {'ssl_listen_port' => 5678}},
       {'certname' => 'replica.dev',
        'tags' => ['puppet_enterprise::profile::primary_master_replica'],
        'title' => 'Puppet_enterprise::Profile::Puppetdb',
        'parameters' => {'ssl_listen_port' => 5678}},
       {'certname' => 'master.dev',
        'tags' => [],
        'title' => 'Puppet_enterprise::Profile::Master',
        'parameters' => {'ssl_listen_port' => 4321}},
       {'certname' => 'master.dev',
        'tags' => [],
        'title' => 'Puppet_enterprise::Master::File_sync',
        'parameters' => {'puppetserver_webserver_ssl_port' => 4321,
                         'compile_master' => false,
                         'replication_mode' => 'source',
                         'storage_service_disabled' => false}},
       {'certname' => 'compile-master.dev',
        'tags' => [],
        'title' => 'Puppet_enterprise::Profile::Master',
        'parameters' => {'ssl_listen_port' => 8140}},
       {'certname' => 'compile-master.dev',
        'tags' => [],
        'title' => 'Puppet_enterprise::Master::File_sync',
        'parameters' => {'puppetserver_webserver_ssl_port' => 8140,
                         'compile_master' => true,
                         'replication_mode' => 'none',
                         'storage_service_disabled' => true}},
       {'certname' => 'master.dev',
        'tags' => [],
        'title' => 'Puppet_enterprise::Profile::Orchestrator',
        'parameters' => {'ssl_listen_port' => 77777,
                         'pcp_listen_port' => 22222,}},
      ]
    end
  end

  let(:node) { 'cli_config.storeconfigstrue.1.test' }

  let(:params) do
    {
      'path'  => 'some-config.hocon',
      'user'  => 'thebestuser',
      'group' => 'theworstgroup',
      'mode'  => '0666',
    }
  end

  it { should contain_pe_hocon_setting('some-config.hocon/services')
               .with_setting('services')
               .with_type('array')
               .with_value([{'type'          => 'code-manager',
                             'url'           => 'https://master.dev:1234/',
                             'server'        => 'master.dev',
                             'port'          => 1234,
                             'prefix'        => '',
                             'status_url'    => 'https://master.dev:4321/status',
                             'status_prefix' => 'status',
                             'status_key'    => 'code-manager-service',
                             'node_certname' => 'master.dev',
                             'display_name'  => 'Code Manager',},

                            {'type'          => 'code-manager',
                             'url'           => 'https://replica.dev:1234/',
                             'server'        => 'replica.dev',
                             'port'          => 1234,
                             'prefix'        => '',
                             'status_url'    => 'https://replica.dev:4321/status',
                             'status_prefix' => 'status',
                             'status_key'    => 'code-manager-service',
                             'node_certname' => 'replica.dev',
                             'display_name'  => 'Code Manager',},

                            {'type'          => 'file-sync-client',
                             'url'           => 'https://replica.dev:4321/',
                             'server'        => 'replica.dev',
                             'port'          => 4321,
                             'prefix'        => '',
                             'status_url'    => 'https://replica.dev:4321/status',
                             'status_prefix' => 'status',
                             'status_key'    => 'file-sync-client-service',
                             'node_certname' => 'replica.dev',
                             'display_name'   => 'File Sync Client Service',},

                            {'type'          => 'puppetdb',
                             'url'           => 'https://master.dev:5678/pdb',
                             'server'        => 'master.dev',
                             'port'          => 5678,
                             'prefix'        => 'pdb',
                             'status_url'    => 'https://master.dev:5678/status',
                             'status_prefix' => 'status',
                             'status_key'    => 'puppetdb-status',
                             'node_certname' => 'master.dev',
                             'display_name'   => 'PuppetDB',},

                            {"type"=>"puppetdb",
                             "url"=>"https://replica.dev:5678/pdb",
                             "server"=>"replica.dev",
                             "port"=>5678,
                             "prefix"=>"pdb",
                             "status_url"=>"https://replica.dev:5678/status",
                             "status_prefix"=>"status",
                             "status_key"=>"puppetdb-status",
                             "node_certname"=>"replica.dev",
                             "display_name"=>"PuppetDB",},

                            {'type'          => 'master',
                             'url'           => 'https://master.dev:4321/',
                             'server'        => 'master.dev',
                             'port'          => 4321,
                             'prefix'        => '',
                             'status_url'    => 'https://master.dev:4321/status',
                             'status_prefix' => 'status',
                             'status_key'    => 'pe-master',
                             'node_certname' => 'master.dev',
                             'display_name'   => 'Puppet Server',},

                            {'type'          => 'file-sync-storage',
                             'url'           => 'https://master.dev:4321/',
                             'server'        => 'master.dev',
                             'port'          => 4321,
                             'prefix'        => '',
                             'status_url'    => 'https://master.dev:4321/status',
                             'status_prefix' => 'status',
                             'status_key'    => 'file-sync-storage-service',
                             'node_certname' => 'master.dev',
                             'display_name'   => 'File Sync Storage Service',},

                            {'type'          => 'file-sync-client',
                             'url'           => 'https://master.dev:4321/',
                             'server'        => 'master.dev',
                             'port'          => 4321,
                             'prefix'        => '',
                             'status_url'    => 'https://master.dev:4321/status',
                             'status_prefix' => 'status',
                             'status_key'    => 'file-sync-client-service',
                             'node_certname' => 'master.dev',
                             'display_name'   => 'File Sync Client Service',},

                            {'type'          => 'master',
                             'url'           => 'https://compile-master.dev:8140/',
                             'server'        => 'compile-master.dev',
                             'port'          => 8140,
                             'prefix'        => '',
                             'status_url'    => 'https://compile-master.dev:8140/status',
                             'status_prefix' => 'status',
                             'status_key'    => 'pe-master',
                             'node_certname' => 'compile-master.dev',
                             'display_name'   => 'Puppet Server',},

                            {'type'          => 'file-sync-client',
                             'url'           => 'https://compile-master.dev:8140/',
                             'server'        => 'compile-master.dev',
                             'port'          => 8140,
                             'prefix'        => '',
                             'status_url'    => 'https://compile-master.dev:8140/status',
                             'status_prefix' => 'status',
                             'status_key'    => 'file-sync-client-service',
                             'node_certname' => 'compile-master.dev',
                             'display_name'   => 'File Sync Client Service',},

                            {'type'          => 'orchestrator',
                             'url'           => 'https://master.dev:77777/orchestrator',
                             'server'        => 'master.dev',
                             'port'          => 77777,
                             'prefix'        => 'orchestrator',
                             'status_url'    => 'https://master.dev:77777/status',
                             'status_prefix' => 'status',
                             'status_key'    => 'orchestrator-service',
                             'node_certname' => 'master.dev',
                             'display_name'   => 'Orchestrator',},

                            {'type'          => 'pcp-broker',
                             'url'           => 'wss://master.dev:22222/pcp',
                             'server'        => 'master.dev',
                             'port'          => 22222,
                             'prefix'        => 'pcp',
                             'status_url'    => 'https://master.dev:77777/status',
                             'status_prefix' => 'status',
                             'status_key'    => 'broker-service',
                             'node_certname' => 'master.dev',
                             'display_name'   => 'PCP Broker',},

                            {'type'          => 'pcp-broker',
                             'url'           => 'wss://master.dev:22222/pcp2',
                             'server'        => 'master.dev',
                             'port'          => 22222,
                             'prefix'        => 'pcp2',
                             'status_url'    => 'https://master.dev:77777/status',
                             'status_prefix' => 'status',
                             'status_key'    => 'broker-service',
                             'node_certname' => 'master.dev',
                             'display_name'   => 'PCP Broker v2',},
                           ])}

  it { should contain_pe_hocon_setting('some-config.hocon/nodes').with_value(
                [
                  {
                    'certname' => 'master.rspec',
                    'role' => 'primary_master',
                    'display_name' => 'Primary Master',
                  },
                  {
                    'certname' => 'replica.dev',
                    'role' => 'primary_master_replica',
                    'display_name' => 'Primary Master Replica',
                  },
                ])}
  it { should contain_pe_hocon_setting('some-config.hocon/certs')
               .with_value({'ca-cert' => '/etc/puppetlabs/puppet/ssl/certs/ca.pem'})}
end
