require 'spec_helper'
describe 'puppet_enterprise::console_services' do
  let(:params) do
    {
      'client_certname' => 'console.rspec',
      'master_host'     => 'master.rspec',
      'orchestrator_port'     => '4246',
      'orchestrator_url_prefix' => '/orchestrator',
      'inventory_url_prefix' => '/inventory',
      'puppetdb_host'           => ['puppetdb.rspec'],
      'puppetdb_port'           => [8081],
    }
  end

  context "when managing default java_args" do
    it { should contain_puppet_enterprise__trapperkeeper__java_args('console-services').with(
      :java_args => { 'Xmx' => '256m',
                      'Xms' => '256m', }
      )
    }
  end

  context "with custom java_args" do
    before(:each) { params['java_args'] = { 'Xms' => '128m', 'Xmx' => '333m' }
                    params['enable_gc_logging'] = true }
    it { should contain_puppet_enterprise__trapperkeeper__java_args('console-services').with(
      :java_args => { 'Xmx' => '333m',
                      'Xms' => '128m', },
      :enable_gc_logging => true,
      )
    }
  end

  context "managing services" do
    it { should contain_service('pe-console-services') }
  end

  context "Using Default parameters" do
    it { should contain_puppet_enterprise__trapperkeeper__console_services('console-services').with(
      :no_longer_reporting_cutoff => nil,
      :query_cache_ttl            => 30000,
      ) }
  end

  context "Passing Custom Parameters" do
    before(:each) do
      params['no_longer_reporting_cutoff'] = 2222
      params['agent_installer_host'] = 'foo'
      params['agent_installer_port'] = 8140
      params['query_cache_ttl'] = 1337
      params['max_connections_per_route'] = 7
      params['max_connections_total'] = 9
    end
    it { should contain_puppet_enterprise__trapperkeeper__console_services('console-services').with(
      :no_longer_reporting_cutoff => 2222,
      :agent_installer_host => 'foo',
      :agent_installer_port => 8140,
      :query_cache_ttl => 1337,
      :max_connections_per_route => 7,
      :max_connections_total => 9,
      ) }
  end

  context "managing service defaults" do
    it { should contain_puppet_enterprise__trapperkeeper__init_defaults('console-services')
      .with({ 'service_stop_retries' => 60,
              'start_timeout'        => 300 })
    }

    context "with overrides" do
      before(:each) do
        params['service_stop_retries'] = 12345
        params['start_timeout'] = 67890
      end

      it { should contain_puppet_enterprise__trapperkeeper__init_defaults('console-services')
        .with({ 'service_stop_retries' => 12345,
                'start_timeout'        => 67890 })
      }
    end
  end
end
