require 'spec_helper'

describe 'puppet_enterprise' do
  it { should compile }

  context 'PE-22434: Hiera warns for parameters not in the module namespace' do
    let(:tempdir) { Dir.mktmpdir('hiera') } # just in case /tmp/hiera.yaml exists...
    # No global Hiera configuration
    let(:hiera_config) { "#{tempdir}/hiera.yaml" }
    # No extra puppet code
    let(:pre_condition) {}
    let(:module_root) { File.join(File.dirname(__FILE__),'..','..') }
    let(:module_hiera_spec_yaml) { "#{module_root}/hiera.spec.yaml" }
    let(:module_hiera_dir) { "#{module_root}/spec/fixtures/hiera/module" }
    let(:module_hiera_datadir) { "#{module_hiera_dir}/conf.d" }

    around(:each) do |example|
      # Temporarily changing the module's hiera.yaml to be hiera.spec.yaml
      # with a modified datadir pointing to spec/fixtures/hiera/module
      module_hiera_yaml = YAML.load_file("#{module_root}/hiera.yaml")
      module_hiera_yaml['defaults']['datadir'] = module_hiera_datadir
      File.write(module_hiera_spec_yaml, module_hiera_yaml.to_yaml)

      FileUtils.mkdir_p(module_hiera_datadir)
      pe_conf = <<-PE_CONF
        "puppet_enterprise::puppet_master_host": "foo"

        # This parameter causes the default module loader to emit a warning:
        # "Module 'puppet_enterprise': Value returned from data_hash function
        # 'hocon_data', when using location
        # '/home/jpartlow/work/src/pl/modules/puppetlabs-puppet_enterprise/spec/classes/../../spec/fixtures/hiera/module/conf.d/pe.conf',
        # must use keys qualified with the name of the module"
        "different_namespace::param": "value"
      PE_CONF
      File.write("#{module_hiera_datadir}/pe.conf", pe_conf)

      begin
        example.run
      ensure
        FileUtils.rm_rf(module_hiera_dir, :secure => true)
        FileUtils.rm(module_hiera_spec_yaml)
        FileUtils.rm_rf(tempdir, :secure => true)
      end
    end

    before(:each) do
      # Fool Puppet into using our temporary hiera.spec.yaml...
      # (We can't stub constants in the around()...)
      stub_const('Puppet::Pops::Lookup::HieraConfig::CONFIG_FILE_NAME', 'hiera.spec.yaml')
    end

    # Note: the @logs array is filled by a Puppet::Test::LogCollector log destination
    # set up by puppetlabs_spec_helper:
    # https://github.com/puppetlabs/puppetlabs_spec_helper/blob/v2.5.1/lib/puppetlabs_spec_helper/puppet_spec_helper.rb#L147
    #
    # The warnings about parameter keys not qualified with the module name bubbles
    # up from here:
    # https://github.com/puppetlabs/puppet/blob/5.3.3/lib/puppet/pops/lookup/module_data_provider.rb#L51
    it 'emits no warnings' do
      catalogue
      expect(@logs.find { |l| l.message =~ /Compiled catalog for.*in.*seconds/ }).to_not be_nil
      warnings = @logs.select { |l| l.level == :warning }.map { |l| l.message }

      expect(warnings).to be_empty
    end
  end
end

