require 'spec_helper'

describe 'puppet_enterprise::license' do
  let(:default_license_path) { '/etc/puppetlabs/license.key' }
  let(:params) do
    {}
  end

  context 'when not managing the license key file' do
    before :each do
      params['manage_license_key'] = false
    end

    it { is_expected.to_not contain_file(default_license_path) }
  end

  context 'user specificed license path' do
    let(:license_key_path) { '/some/nonstandard/spot' }
    let(:pre_condition) do
      <<-PRE_COND
      class { 'puppet_enterprise':
        puppet_master_host => 'master.rspec',
        license_key_path => '#{license_key_path}',
      }
      PRE_COND
    end

    it 'manages the license and ensures the provided content' do
      mock_puppet_file(license_key_path, 'fake license key')
      is_expected.to contain_file(default_license_path)
        .with_content('fake license key')
    end
  end

  context 'default license path' do
    context 'with no license' do
      it { is_expected.to_not contain_file(default_license_path) }
    end

    context 'with a license' do
      # bust catalog cache
      let(:node) { 'master-with-managed-license.rspec' }

      it 'manages the license' do
        mock_puppet_file(default_license_path, 'license')
        is_expected.to contain_file(default_license_path)
          .with_content('license')
      end
    end
  end
end
