require 'spec_helper'

describe 'puppet_enterprise::master::pcp_broker' do
  let(:layout) { mono }
  let(:node) { 'compile.rspec' }
  let(:params) {{ :orchestrator_hosts => ['orch.rspec', 'replica.rspec'] }}

  let(:pre_condition) do
    <<-PRE_COND
      class {'puppet_enterprise':
        puppet_master_host           => 'master.rspec',
      }
      service { 'pe-puppetserver': }
      package { 'pe-puppetserver': }
    PRE_COND
  end

  context "configuring the webserver" do
    it { should contain_puppet_enterprise__trapperkeeper__webserver_settings("pcp-broker")
      .with('container'          => 'puppetserver',
            'ssl_listen_address' => '0.0.0.0',
            'ssl_listen_port'    => 8142,
            'ssl_protocols'      => ['TLSv1.2'],
            )
    }
    it { should contain_pe_hocon_setting('puppetserver.web-router-service.broker-service').with(
        :setting => 'web-router-service."puppetlabs.pcp.broker.service/broker-service"',
        :value => {
          'v1' => {
            'route' => '/pcp',
            'server' => 'pcp-broker'
          },
          'v2' => {
            'route' => '/pcp2',
            'server' => 'pcp-broker'
          },
          'metrics' => {
            'route' => '/',
            'server' => 'pcp-broker'
          }
        })
    }
  end

  context "configuring pcp-broker" do
      it { should contain_pe_hocon_setting('puppetserver.pcp-broker.controller-uris').with_value(['wss://orch.rspec:8143/server', 'wss://replica.rspec:8143/server']).with_type('array') }
  end
end
