require 'spec_helper'

describe 'puppet_enterprise::master::recover_configuration' do
  let(:cronname) { 'puppet infra recover_configuration' }
  let(:facts) do 
    { }
  end
  let(:params) do
    {
      'recover_configuration_interval_offset' => 7
    }
  end

    context "recover_configuration cron job with defaults" do
      it { should contain_cron(cronname).with(
        'user'   => 'root',
        'minute' => [ '7', '37'],
      )}
    end
    context "recover_configuration cron job with specified interval" do
      before :each do
        params['recover_configuration_interval'] = 20
      end
      it { should contain_cron(cronname).with(
        'user'   => 'root',
        'minute' => [ '7', '27', '47', ],
      )}
    end
    context "recover_configuration cron job is absent when recover_configuration_interval is 0" do
      let(:params) do
        {
          'recover_configuration_interval' => 0
        }
      end
      it { should contain_cron(cronname).with(
        'ensure' => 'absent',
      ) }
    end

    context "catalog fails when interval_offset is > 59" do
      before :each do
        params['recover_configuration_interval_offset'] = 60
      end
      it "fails to compile" do
        expect { catalogue }.to raise_error(Puppet::PreformattedError, /expects an Integer\[0, 59\]/)
      end
    end
end
