require 'spec_helper'

describe 'puppet_enterprise::profile::console::cache' do
  let(:params) do
    {}
  end

  let(:config_dir) { '/etc/puppetlabs/console-services/conf.d' }
  let(:cache_conf_file) { "#{config_dir}/cache.conf" }

  let(:pre_condition) do
    <<-PRE_COND
      service { 'pe-console-services': }
      package { 'pe-console-services': }
    PRE_COND
  end

  context "when default parameters are used" do
    it { should contain_file(cache_conf_file).with(
      :ensure => "file",
      :owner  => "pe-console-services",
      :group  => "pe-console-services",
      :mode   => "0640") }

     it { should contain_pe_hocon_setting('console-services.console.cache-size-bytes').
       with_ensure('absent') }

     it { should contain_pe_hocon_setting('console-services.console.cache-whitelist').
       with_ensure('present').
       with_value( {}) }
  end

  context "when parameters are provided " do
    before :each do
      params['cache_size_bytes'] = 1234
      params['cache_whitelist'] = {
                                    "^/api/cm/facts" => { "ttl" => 3600 },
                                    "^/api/cm/nodes\\??.*" => { "ttl" => 3600 },
                                    "^/api/cm/node-counts\\??.*" => { "ttl" => 3600 }
                                  }
    end
     it { should contain_pe_hocon_setting('console-services.console.cache-size-bytes').
       with_ensure('present').
       with_value( 1234 ) }

     it { should contain_pe_hocon_setting('console-services.console.cache-whitelist').
       with_ensure('present').
       with_value( {
                     "^/api/cm/facts" => { "ttl" => 3600 },
                     "^/api/cm/nodes\\??.*" => { "ttl" => 3600 },
                     "^/api/cm/node-counts\\??.*" => { "ttl" => 3600 }
                   }) }
  end
end
