require 'spec_helper'

describe 'puppet_enterprise::profile::console::certs' do
  let(:params) do
    { 'certname' => 'console-rspec' }
  end
  let(:services_sharedir) { '/opt/puppetlabs/server/data/console-services' }
  let(:ssldir) { '/etc/puppetlabs/puppet/ssl' }

  let(:pre_condition) do
    <<-PRE_COND
      class {'puppet_enterprise':
        puppet_master_host           => 'master.rspec',
      }
      class {'puppet_enterprise::console_services':
        master_host           => 'master.rspec',
        client_certname       => 'master.rspec',
        puppetdb_host         => ['puppetdb.rspec'],
        puppetdb_port         => [8081],
      }
      file { '/etc/puppetlabs/console-services/conf.d/webserver.conf': }
    PRE_COND
  end

  context "managing certs" do
    it { should contain_file("#{services_sharedir}/certs").with_mode('0600') }
    it {
      should contain_file("#{services_sharedir}/certs/console-rspec.cert.pem").with(
        :source => "#{ssldir}/certs/console-rspec.pem",
      )
    }

    it {
      should contain_file("#{services_sharedir}/certs/console-rspec.private_key.pk8").with(
               :owner => "pe-console-services",
               :group => "pe-console-services",
               :mode => "0400"
             )
    }

  end
end
