require 'spec_helper'

describe 'puppet_enterprise::profile::console::proxy::http_redirect' do
  let(:params) do
    { 'ssl_listen_port' => 443 }
  end

  let(:pre_condition) do
    <<-PRE_COND
      class {'puppet_enterprise':
        puppet_master_host => 'master.rspec',
      }
      service { 'pe-nginx': }
    PRE_COND
  end

  def should_have_directive(name, value)
    should contain_pe_nginx__directive(name).with_value(value).with('directive_ensure' => 'present')
  end

  context 'enable http_redirect = true' do
    before :each do
      params['enable_http_redirect'] = true
    end

    it do
      should contain_file('/etc/puppetlabs/nginx/conf.d/http_redirect.conf').
        with('ensure'  => 'file').
        with_content(%r{server_name master\.rspec}).
        that_notifies('Service[pe-nginx]')
    end
  end

  context 'with custom ssl_listen_port' do
    before :each do
      params['enable_http_redirect'] = true
      params['ssl_listen_port'] = 5050
    end

    it do
      should contain_file('/etc/puppetlabs/nginx/conf.d/http_redirect.conf').
        with('ensure'  => 'file').
        with_content(%r{301 https://\$server_name\$request_uri:5050})
    end
  end

  context 'with custom server_name' do
    before :each do
      params['server_name'] = 'my_custom_server_name'
    end

    it do
      should contain_file('/etc/puppetlabs/nginx/conf.d/http_redirect.conf').
        with('ensure'  => 'file').
        with_content(%r{server_name my_custom_server_name})
    end
  end

  context 'enable http_redirect = false' do
    before :each do
      params['enable_http_redirect'] = false
    end
    it { should contain_file('/etc/puppetlabs/nginx/conf.d/http_redirect.conf').with('ensure' => 'absent') }
  end
end
