require 'spec_helper'

describe 'puppet_enterprise::profile::master::auth_conf' do

  let(:params) do
    {
      'console_client_certname'      => 'fake_console_certname',
      'classifier_client_certname'   => 'fake_classifier_certname',
      'orchestrator_client_certname' => 'fake_orchestrator_certname',
      'allow_unauthenticated_ca'     => true,
      'allow_unauthenticated_status' => true
    }
  end

  let(:pre_condition) do
    <<-PRE_COND
      class {'puppet_enterprise':
        puppet_master_host           => 'master.rspec',
      }
      package { 'pe-puppetserver': }
      service { 'pe-puppetserver': }
    PRE_COND
  end

  context 'managing legacy Ruby auth.conf' do
    context 'when not custom auth.conf' do
      let(:facts) {{ 'custom_auth_conf' => false }}
      it { should_not contain_file('/etc/puppetlabs/puppet/auth.conf') }
    end

    context 'when custom auth.conf' do
      let(:facts) {{ 'custom_auth_conf' => true }}
      it { should_not contain_file('/etc/puppetlabs/puppet/auth.conf') }
    end
  end

  context 'managing new Trapperkeeper auth.conf' do
    it { should contain_class('puppet_enterprise::master::tk_authz')
                 .with_console_client_certname('fake_console_certname')
                 .with_classifier_client_certname('fake_classifier_certname')
                 .with_orchestrator_client_certname('fake_orchestrator_certname')
                 .with_require('Package[pe-puppetserver]')
                 .with_notify('Service[pe-puppetserver]') }
  end

  context 'when Trapperkeeper auth.conf requires authentication for CA endpoints' do
    let(:params) do
      {
        'console_client_certname'      => 'fake_console_certname',
        'classifier_client_certname'   => 'fake_classifier_certname',
        'orchestrator_client_certname' => 'fake_orchestrator_certname',
        'allow_unauthenticated_ca'     => false,
        'allow_unauthenticated_status' => true
      }
    end
    it { should contain_pe_puppet_authorization__rule('puppetlabs certificate')
                  .with_allow_unauthenticated(false)
                  .with_allow('*') }
    it { should contain_pe_puppet_authorization__rule('puppetlabs crl')
              .with_allow_unauthenticated(false)
              .with_allow('*') }
    it { should contain_pe_puppet_authorization__rule('puppetlabs csr')
              .with_allow_unauthenticated(false)
              .with_allow('*') }
  end

  context 'when Trapperkeeper auth.conf requires authentication for puppetserver status-service endpoint' do
    let(:params) do
      {
        'console_client_certname'      => 'fake_console_certname',
        'classifier_client_certname'   => 'fake_classifier_certname',
        'orchestrator_client_certname' => 'fake_orchestrator_certname',
        'allow_unauthenticated_ca'     => true,
        'allow_unauthenticated_status' => false
      }
    end

    it { should contain_pe_puppet_authorization__rule('puppetserver simple status endpoint')
              .with_allow_unauthenticated(true) }

    it { should contain_pe_puppet_authorization__rule('puppetlabs status service')
              .with_allow_unauthenticated(false)
              .with_allow('*') }
  end
end
