require 'spec_helper'

describe 'puppet_enterprise::profile::master::classifier' do
  let(:params) do
    {
      'classifier_host'      => 'classifier.rspec',
      'classifier_url_prefix'  => '/classifier-api',
      'classifier_port' => '1234',
    }
  end
  let(:config_dir) { "/etc/puppetlabs/puppet" }

  context "managing puppet.conf" do
    it { should contain_pe_ini_setting('node_terminus').with_value('classifier') }
  end

  context "managing puppet to talk to the classifier" do
    context "With a single host" do
      it { should contain_file("#{config_dir}/classifier.yaml").with_content( <<END
- server: classifier.rspec
  port: 1234
  prefix: /classifier-api
END
                                                                         )}
    end

    context "With multiple hosts" do
      let(:params) {
        { 'classifier_host'      => ['classifier.rspec', 'fallback.rspec'],
          'classifier_url_prefix'  => '/classifier-api',
          'classifier_port' => ['1234', '1234'] }
      }

      it { should contain_file("#{config_dir}/classifier.yaml").with_content( <<END
- server: classifier.rspec
  port: 1234
  prefix: /classifier-api
- server: fallback.rspec
  port: 1234
  prefix: /classifier-api
END
                                                                         )}
    end
  end
  context "specifying a custom node_terminus" do
    let(:params) do
      { 'classifier_host'       => 'classifier.rspec',
                     'classifier_url_prefix' => '/classifier-api',
                     'classifier_port'       => '1234',
                     'node_terminus'         => 'custom',
                   }
    end
    it { should contain_pe_ini_setting('node_terminus').with_value('custom') }
  end
end
