require 'spec_helper'

describe 'puppet_enterprise::profile::primary_master_replica' do
  let(:node) { 'replica.rspec' }
  let(:facts) do
    {
      'fqdn'              => node,
      'clientcert'        => node,
      'custom_auth_conf'  => true,
      'aio_agent_build'   => '1.3.3',
      'processors'        => {"count"=>8, "speed"=>"2.5 GHz"},
    }
  end

  before :each do
    Puppet::Parser::Functions.newfunction(:pe_compiling_server_aio_build, :type => :rvalue) do |args|
      '1.3.3'
    end
  end

  it { should satisfy_all_relationships }

  it 'is a database configured to reference itself as a MoM' do
    should contain_class('puppet_enterprise::profile::database')
             .with_certname('replica.rspec')
  end

  it 'is a master configured to reference itself as a MoM' do
    should contain_class('puppet_enterprise::profile::master')
      .with_certname('replica.rspec')
      .with_classifier_host('replica.rspec')
      .with_console_host('replica.rspec')
      .with_console_server_certname('replica.rspec')
      .with_puppetdb_host(['puppetdb.rspec', 'replica.rspec'])
      .with_ca_host('master.rspec') # CA traffic is proxied.
      .with_master_of_masters_certname('replica.rspec')
  end

  it 'is a puppetdb configured to reference itself as a MoM' do
    should contain_class('puppet_enterprise::profile::puppetdb')
             .with_certname('replica.rspec')
             .with_database_host('replica.rspec')
             .with_master_certname('replica.rspec')
             .with_rbac_host('replica.rspec')
  end

  it 'is a console configured to reference itself as a MoM' do
    should contain_class('puppet_enterprise::profile::console')
             .with_certname('replica.rspec')
             .with_database_host('replica.rspec')
             .with_master_host('replica.rspec')
             .with_master_certname('replica.rspec')
             .with_puppetdb_host(['replica.rspec'])
             .with_ca_host('replica.rspec')
  end

  it 'installs the puppet-infrastructure face shims' do
    should contain_class('pe_infrastructure::puppet_infra_shims')
  end

  context 'using an external Postgres replica' do
    let(:params) do
      { 'database_host_puppetdb' => 'external.replica' }
    end

    it 'is configured to reference the external postgres node for the PuppetDB database_host' do
      should contain_class('puppet_enterprise::profile::puppetdb')
               .with_database_host('external.replica')
    end
  end
end
