require 'spec_helper'

describe 'puppet_enterprise::puppetdb::config_ini', :type => :class do
  let(:params) do
    {
      'confdir'                    => '/etc/puppetlabs/puppetdb/conf.d',
      'command_processing_threads' => 4,
      'concurrent_writes'          => 4,
    }
  end

  let(:pre_condition) do
    <<-PRE_COND
      class {'puppet_enterprise':
        puppet_master_host           => 'master.rspec',
      }
      service { 'pe-puppetdb': }
    PRE_COND
  end

  it { should contain_class('puppet_enterprise::puppetdb::config_ini') }
  it { should contain_file('/etc/puppetlabs/puppetdb/conf.d/config.ini') }

  describe 'when using default values' do
    it { should contain_pe_ini_setting('config.ini threads command-processing section').
         with(
           'ensure'  => 'present',
           'path'    => '/etc/puppetlabs/puppetdb/conf.d/config.ini',
           'section' => 'command-processing',
           'setting' => 'threads',
           'value'   => 4
    )}
    it { should contain_pe_ini_setting('config.ini concurrent-writes command-processing section').
         with(
           'ensure'  => 'present',
           'path'    => '/etc/puppetlabs/puppetdb/conf.d/config.ini',
           'section' => 'command-processing',
           'setting' => 'concurrent-writes',
           'value'   => 4
    )}
  end
  describe 'when using 0s for command-processing threads & writes expect 1s' do
    before :each do
      params['command_processing_threads'] = 0
      params['concurrent_writes'] = 0
    end
      it { should contain_pe_ini_setting('config.ini threads command-processing section').
         with(
           'ensure'  => 'present',
           'path'    => '/etc/puppetlabs/puppetdb/conf.d/config.ini',
           'section' => 'command-processing',
           'setting' => 'threads',
           'value'   => 1
      )}
      it { should contain_pe_ini_setting('config.ini concurrent-writes command-processing section').
         with(
           'ensure'  => 'present',
           'path'    => '/etc/puppetlabs/puppetdb/conf.d/config.ini',
           'section' => 'command-processing',
           'setting' => 'concurrent-writes',
           'value'   => 1
      )}
  end
end
