require 'spec_helper'

describe 'puppet_enterprise::puppetdb::sync_ini' do
  let(:params) do
    {
      'peers' => [{'host' => 'remote1',
                   'port' => 1234,
                   'sync_interval_minutes' => 2},
                  {'host' => 'remote2',
                   'port' => 5678,
                   'sync_interval_minutes' => 42}]
    }
  end

  let(:pre_condition) do
    <<-PRE_COND
      class {'puppet_enterprise':
        puppet_master_host           => 'master.rspec',
      }
      service { 'pe-puppetdb': }
    PRE_COND
  end

  it { should contain_class('puppet_enterprise::puppetdb::sync_ini') }
  it { should contain_file('/etc/puppetlabs/puppetdb/conf.d/sync.ini') }

  it { should contain_pe_ini_setting('puppetdb_sync_server_urls').
               with('ensure' => 'present',
                    'path' => '/etc/puppetlabs/puppetdb/conf.d/sync.ini',
                    'section' => 'sync',
                    'value' => 'https://remote1:1234,https://remote2:5678',
                    'setting' => 'server_urls')}

  it { should contain_pe_ini_setting('puppetdb_sync_intervals').
               with('ensure'  => 'present',
                    'path'    => '/etc/puppetlabs/puppetdb/conf.d/sync.ini',
                    'section' => 'sync',
                    'setting' => 'intervals',
                    'value'   => '2m,42m')}
end
