require 'spec_helper'

describe 'puppet_enterprise::trapperkeeper::analytics' do
  let(:facts) { {clientcert: 'zombo.com'} }
  let(:datadir) { "/my/data/dir" }
  let(:params) do
    {
      container: 'puppetserver',
      datadir: datadir,
    }
  end
  let(:product_name) do
    { "artifact-id" => "pe-master",
      "group-id"    => "puppetlabs.packages" }
  end
  let(:pre_condition) do
    <<-PRE_COND
      class {'puppet_enterprise':
        puppet_master_host           => 'master.rspec',
      }
      service { 'pe-puppetserver': }
    PRE_COND
  end

  context 'HOCON' do
    it { should contain_puppet_enterprise__trapperkeeper__bootstrap_cfg('analytics-service')
                  .with_container('puppetserver')
                  .with_namespace('puppetlabs.analytics.service') }

    it { should contain_file('/etc/puppetlabs/puppetserver/conf.d/analytics.conf')
                  .with_owner('pe-puppet')
                  .with_group('pe-puppet')
                  .with_mode('0640') }

    it { should contain_pe_hocon_setting('analytics.data-directory')
                  .with_path('/etc/puppetlabs/puppetserver/conf.d/analytics.conf')
                  .with_value(datadir) }

    it { should contain_file('/etc/puppetlabs/puppetserver/conf.d/product.conf')
                  .with_owner('pe-puppet')
                  .with_group('pe-puppet')
                  .with_mode('0640') }

    it { should contain_pe_hocon_setting('product.version-path')
                  .with_path('/etc/puppetlabs/puppetserver/conf.d/product.conf')
                  .with_value('/opt/puppetlabs/server/pe_version') }

    it { should contain_pe_hocon_setting('product.name')
                  .with_path('/etc/puppetlabs/puppetserver/conf.d/product.conf')
                  .with_value(product_name) }
  end

  it { should contain_file(datadir)
                .with_mode('0640')
                .with_owner('pe-puppet')
                .with_group('pe-puppet') }

  context 'webserver' do
    it { should contain_pe_hocon_setting("puppetserver.webserver.analytics")
                  .with_ensure('absent')
                  .with_path('/etc/puppetlabs/puppetserver/conf.d/webserver.conf')
                  .with_setting('webserver.analytics')
    }
  end
end
