require 'spec_helper'

describe 'puppet_enterprise::trapperkeeper::inventory' do
  let(:params) do
    {
      'puppetdb_url' => 'https://puppetdb.rspec:8081',
      'client_cert'  => '/etc/puppetlabs/orchestration-services/ssl/master.rspec.cert.pem',
      'client_pem_key'  => '/etc/puppetlabs/orchestration-services/ssl/master.rspec.private_key.pem',
    }
  end
  let(:container) { 'orchestration-services' }
  let(:config_dir) { "/etc/puppetlabs/#{container}" }
  let(:secrets_dir) { "#{config_dir}/conf.d/secrets" }
  let(:service) { 'Service[pe-orchestration-services]' }

  let(:pre_condition) do
    <<-PRE_COND
      class {'puppet_enterprise':
        puppet_master_host           => 'master.rspec',
      }
      service { 'pe-orchestration-services': }
      package { 'pe-orchestration-services': }
    PRE_COND
  end

  it { should contain_file("#{config_dir}/conf.d/inventory.conf").with(:owner => 'pe-orchestration-services',
                                                                       :group => 'pe-orchestration-services',
                                                                       :mode  => '0640') }

  context "configuring the SSL client" do
    it { should contain_pe_hocon_setting("#{container}.inventory.ssl-cert").with_value("#{config_dir}/ssl/master.rspec.cert.pem").with_notify(service) }
    it { should contain_pe_hocon_setting("#{container}.inventory.ssl-key").with_value("#{config_dir}/ssl/master.rspec.private_key.pem").with_notify(service) }
    it { should contain_pe_hocon_setting("#{container}.inventory.ssl-ca-cert").with_value("/etc/puppetlabs/puppet/ssl/certs/ca.pem").with_notify(service) }
  end

  context "managing integration with other services" do
    it { should contain_pe_hocon_setting("#{container}.inventory.puppetdb-url").with_value('https://puppetdb.rspec:8081').with_notify(service) }
  end
end
