require 'spec_helper'

describe 'puppet_enterprise::file_sync_directory' do
  let(:title) { 'namevar' }

  let(:file_sync_conf) { '/etc/puppetlabs/puppetserver/conf.d/file-sync.conf' }

  let(:params) do
    { 
      'live_dir'              => '/opt/puppetlabs/server/data/primary-ca-data',
      'staging_dir'           => '/etc/puppetlabs/puppet/ssl',
      'file_sync_config_file' => file_sync_conf,
      'file_sync_repo_name'   => 'ca-data',
      'replication_mode'      => 'none',
    }
  end

  on_supported_os.each do |os, os_facts|
    context "on #{os}" do
      let(:facts) { os_facts }

      it { is_expected.to compile }
    end
  end

  context 'replication enabled' do
    context "source" do
      before :each do
        params['replication_mode'] = 'source'
      end

      let(:pre_condition) do
        <<-PRE_COND
          service { 'pe-puppetserver': }
        PRE_COND
      end

      it { should_not contain_pe_hocon_setting('file-sync.repos.ca-data')
                   .with_ensure('absent')
                   .with_setting('file-sync.repos.ca-data')
                   .with_path(file_sync_conf) }
      it { should contain_pe_hocon_setting('file-sync.repos.ca-data.staging-dir')
                   .with_ensure('present')
                   .with_path(file_sync_conf)
                   .with_setting('file-sync.repos.ca-data.staging-dir')
                   .with_value('/etc/puppetlabs/puppet/ssl') }
      it { should contain_pe_hocon_setting('file-sync.repos.ca-data.client-active')
                   .with_ensure('present')
                   .with_setting('file-sync.repos.ca-data.client-active')
                   .with_path(file_sync_conf)
                   .with_value('false') }
      it { should contain_pe_hocon_setting('file-sync.repos.ca-data.honor-gitignore')
                   .with_ensure('present')
                   .with_setting('file-sync.repos.ca-data.honor-gitignore')
                   .with_path(file_sync_conf)
                   .with_value('true') }
      it { should contain_pe_hocon_setting('file-sync.repos.ca-data.auto-commit')
                   .with_ensure('present')
                   .with_setting('file-sync.repos.ca-data.auto-commit')
                   .with_path(file_sync_conf)
                   .with_value('true') }
      it { should contain_file('/etc/puppetlabs/puppet/ssl/.gitignore')
                   .with_ensure('present')
                   .with_mode('0600')
                   .with_source('puppet:///modules/puppet_enterprise/master/file_sync/ca-data/gitignore') }
      it { should contain_pe_hocon_setting('file-sync.repos.ca-data.live-dir')
                   .with_ensure('absent')
                   .with_setting('file-sync.repos.ca-data.live-dir')
                   .with_path(file_sync_conf) }
    end

    context "replica" do
      before :each do
        params['replication_mode'] = 'replica'
      end

      it { should_not contain_pe_hocon_setting('file-sync.repos.ca-data')
                   .with_ensure('absent')
                   .with_setting('file-sync.repos.ca-data')
                   .with_path(file_sync_conf) }
      it { should contain_pe_hocon_setting('file-sync.repos.ca-data.staging-dir')
                   .with_ensure('absent')
                   .with_setting('file-sync.repos.ca-data.staging-dir')
                   .with_path(file_sync_conf) }
      it { should contain_pe_hocon_setting('file-sync.repos.ca-data.client-active')
                   .with_ensure('absent')
                   .with_setting('file-sync.repos.ca-data.client-active')
                   .with_path(file_sync_conf)
                   .with_value('false') }
      it { should contain_pe_hocon_setting('file-sync.repos.ca-data.honor-gitignore')
                   .with_ensure('absent')
                   .with_setting('file-sync.repos.ca-data.honor-gitignore')
                   .with_path(file_sync_conf)
                   .with_value('true') }
      it { should contain_pe_hocon_setting('file-sync.repos.ca-data.auto-commit')
                   .with_ensure('absent')
                   .with_setting('file-sync.repos.ca-data.auto-commit')
                   .with_path(file_sync_conf)
                   .with_value('true')}
      it { should contain_pe_hocon_setting('file-sync.repos.ca-data.live-dir')
                   .with_ensure('present')
                   .with_setting('file-sync.repos.ca-data.live-dir')
                   .with_path(file_sync_conf)
                   .with_value('/opt/puppetlabs/server/data/primary-ca-data') }
    end

    context "none" do
      before :each do
        params['replication_mode'] = 'none'
      end

      it { should contain_pe_hocon_setting('file-sync.repos.ca-data')
                   .with_ensure('absent')
                   .with_setting('file-sync.repos.ca-data')
                   .with_path(file_sync_conf) }
      it { should contain_pe_hocon_setting('file-sync.repos.ca-data.staging-dir')
                   .with_ensure('absent')
                   .with_setting('file-sync.repos.ca-data.staging-dir')
                   .with_path(file_sync_conf) }
      it { should contain_pe_hocon_setting('file-sync.repos.ca-data.client-active')
                   .with_ensure('absent')
                   .with_setting('file-sync.repos.ca-data.client-active')
                   .with_path(file_sync_conf)
                   .with_value('false') }
      it { should contain_pe_hocon_setting('file-sync.repos.ca-data.honor-gitignore')
                   .with_ensure('absent')
                   .with_setting('file-sync.repos.ca-data.honor-gitignore')
                   .with_path(file_sync_conf)
                   .with_value('true') }
      it { should contain_pe_hocon_setting('file-sync.repos.ca-data.auto-commit')
                   .with_ensure('absent')
                   .with_setting('file-sync.repos.ca-data.auto-commit')
                   .with_path(file_sync_conf)
                   .with_value('true')}
      it { should contain_pe_hocon_setting('file-sync.repos.ca-data.live-dir')
                   .with_ensure('absent')
                   .with_setting('file-sync.repos.ca-data.live-dir')
                   .with_path(file_sync_conf) }
    end
  end
end
