require 'spec_helper'

describe 'puppet_enterprise::orchestrator::ruby_service', :type => :define do
  let(:facts) { {} }
  let(:title) { 'abcdefg' }
  let(:port) { 12345}
  let(:params) do
    {   
      'certname' => 'orchestrator.rspec',
      'container' => 'abcdefg',
      'ssl_listen_port' => port,
      'settings' => {'extra' => "extra param"},
    }   
  end 
  let(:ciphers) { ['ECDHE-ECDSA-AES256-GCM-SHA384',
                   'ECDHE-RSA-AES256-GCM-SHA384',
                   'ECDHE-ECDSA-CHACHA20-POLY1305',
                   'ECDHE-RSA-CHACHA20-POLY1305',
                   'ECDHE-ECDSA-AES128-GCM-SHA256',
                   'ECDHE-RSA-AES128-GCM-SHA256',
                   'ECDHE-ECDSA-AES256-SHA384',
                   'ECDHE-RSA-AES256-SHA384',
                   'ECDHE-ECDSA-AES128-SHA256',
                   'ECDHE-RSA-AES128-SHA256'
  ] }
 
  it { is_expected.to compile.with_all_deps }

  context "with valid parameters" do
    context "it should be configured" do
      it { should contain_file("/etc/puppetlabs/abcdefg/conf.d/abcdefg.conf") }
      it { should contain_pe_hocon_setting("abcdefg.host").with(
        :setting => "abcdefg.host",
        :value => "0.0.0.0")
      }   
      it { should contain_pe_hocon_setting("abcdefg.port").with(
        :setting => "abcdefg.port",
        :value => port)
      }
      it { should contain_pe_hocon_setting("abcdefg.logfile").with(
        :setting => "abcdefg.logfile",
        :value => "/var/log/puppetlabs/abcdefg/abcdefg.log")
      }
      it { should contain_pe_hocon_setting("abcdefg.loglevel").with(
        :setting => "abcdefg.loglevel",
        :value => "notice")
      }
      it { should contain_pe_hocon_setting("abcdefg.whitelist").with(
        :setting => "abcdefg.whitelist",
        :type => 'array',
        :value => ["orchestrator.rspec"])
      }
      it { should contain_pe_hocon_setting("abcdefg.ssl_cipher_suites").with(
        :setting => "abcdefg.ssl_cipher_suites",
        :type => 'array',
        :value => ciphers)
      }
      it { should contain_pe_hocon_setting("abcdefg.extra").with(
        :setting => "abcdefg.extra",
        :value => "extra param")
      }

      it { should contain_puppet_enterprise__certs('abcdefg').with(:certname => 'orchestrator.rspec',
                                                                   :container => 'abcdefg') }
      it { should contain_service('pe-abcdefg').with(:ensure => 'running', :enable => true) }
    end
  end
end