require 'spec_helper'

describe 'puppet_enterprise::trapperkeeper::activity' do
  let(:params) do
    {
      'rbac_url_prefix' => '/blah-api',
      'rbac_port'       => '1111',
      'database_user'   => 'activityuser',
    }
  end
  let(:title) { 'console-services' }

  let(:pre_condition) do
    <<-PRE_COND
      class {'puppet_enterprise':
        puppet_master_host           => 'master.rspec',
      }
      service { 'pe-console-services': }
      package { 'pe-console-services': }
    PRE_COND
  end

  context "when the parameters are valid" do
    it do
      should contain_file("/etc/puppetlabs/console-services/conf.d/activity.conf").with(
        :owner => "pe-console-services",
        :group => "pe-console-services",
        :mode => "0640"
      )
    end
    it { should contain_pe_hocon_setting('activity.rbac-base-url').with_value('http://localhost:1111/blah-api/v1') }
    it { should contain_pe_hocon_setting('activity.cors-origin-pattern').with_value('.*') }
    it { should contain_pe_concat__fragment('console-services activity-service') }
    it { should contain_pe_concat__fragment('console-services jetty9-service') }

    it { should contain_puppet_enterprise__trapperkeeper__database_settings('activity').with(
      'container'           => 'console-services',
      'database_host'       => 'localhost',
      'database_name'       => 'pe-activity',
      'database_password'   => nil,
      'database_port'       => '5432',
      'database_properties' => '',
      'database_user'       => 'activityuser',
      'group'               => 'pe-console-services',
      'user'                => 'pe-console-services',
    ) }
  end
end
