require 'spec_helper'

describe 'puppet_enterprise::trapperkeeper::bootstrap_cfg' do
  let(:facts) { {} }
  let(:params) do
    {
      'namespace' => 'puppetlabs.trapperkeeper.services.scheduler.scheduler-service',
      'container' => 'puppetdb',
    }
  end
  let(:title) { 'pe-puppetdb' }

  let(:pre_condition) do
    <<-PRE_COND
      class {'puppet_enterprise':
        puppet_master_host           => 'master.rspec',
      }
      package { 'pe-puppetdb': }
      puppet_enterprise::trapperkeeper::pe_service { 'puppetdb': }
    PRE_COND
  end

  context "when on version above the LTS" do
    before(:each) do
      facts['pe_build'] = '2017.1.1'
    end

    it { should contain_pe_concat("/etc/puppetlabs/puppetdb/bootstrap.cfg").with(
      'notify' => 'Exec[pe-puppetdb service full restart]',
    ) }

  end
  context "when on the LTS" do
    before(:each) do
      facts['pe_build'] = '2016.4.4'
    end

    it { should contain_pe_concat("/etc/puppetlabs/puppetdb/bootstrap.cfg").with(
      'notify' => 'Service[pe-puppetdb]',
    ) }
  end
  context "when on version above the LTS and setting up a compile master " do
    before(:each) do
      # Update facts so the the updated function is recognized for subsequent tests.
      # I think it's because rspec-puppet has an @@cache based on args, and
      # facts are one of the arguments.  So by mutating facts, we are forcing
      # it get a new catalog for evaluation and the overridden function is  used...
      facts['foo'] = 'baz'
      facts['pe_build'] = nil
      Puppet::Parser::Functions.newfunction(:pe_compiling_server_version, :type => :rvalue) do |args|
        '2017.1.1'
      end
      Puppet::Parser::Functions.newfunction(:pe_compile_master, :type => :rvalue) do |args|
        true
      end
    end

    it { should contain_pe_concat("/etc/puppetlabs/puppetdb/bootstrap.cfg").with(
      'notify' => 'Exec[pe-puppetdb service full restart]',
    ) }

  end
  context "when on the LTS and setting up a compile master" do
    before(:each) do
      # Update facts so the the updated function is recognized for subsequent tests.
      # I think it's because rspec-puppet has an @@cache based on args, and
      # facts are one of the arguments.  So by mutating facts, we are forcing
      # it get a new catalog for evaluation and the overridden function is  used...
      facts['foo'] = 'bar'
      facts['pe_build'] = nil
      Puppet::Parser::Functions.newfunction(:pe_compiling_server_version, :type => :rvalue) do |args|
        '2016.4.4'
      end
      Puppet::Parser::Functions.newfunction(:pe_compile_master, :type => :rvalue) do |args|
        true
      end
    end

    it { should contain_pe_concat("/etc/puppetlabs/puppetdb/bootstrap.cfg").with(
      'notify' => 'Service[pe-puppetdb]',
    ) }
  end
end
