require 'spec_helper'

describe 'puppet_enterprise::trapperkeeper::database_settings' do
  let(:params) do
    {
      'container'           => 'console-services',
      'database_host'       => 'db.rspec',
      'database_port'       => 54321,
      'database_user'       => 'pe-rbac',
      'database_properties' => '',
      'database_password'   => '',
      'migration_user'      => '',
      'migration_password'  => '',
      'database_name'       => 'pe-rbac',
      'group'               => 'pe-rbac',
      'user'                => 'pe-rbac',
    }
  end
  let(:title)          { 'rbac' }
  let(:pre_condition) do
    <<-PRE_COND
      class {'puppet_enterprise':
        puppet_master_host           => 'master.rspec',
      }
      service { 'pe-console-services': }
    PRE_COND
  end


  context "when the parameters are valid" do
    it do
      should contain_file('/etc/puppetlabs/console-services/conf.d/rbac-database.conf').with(
        :owner => 'pe-rbac',
        :group => 'pe-rbac',
        :mode  => '0640'
      )
    end

    it { should contain_pe_hocon_setting('console-services.rbac.database.subprotocol')
      .with_value('postgresql')
      .with_notify('Service[pe-console-services]') }
    it { should contain_pe_hocon_setting('console-services.rbac.database.subname')
      .with_value('//db.rspec:54321/pe-rbac')
      .with_notify('Service[pe-console-services]') }
    it { should contain_pe_hocon_setting('console-services.rbac.database.user')
      .with_value('pe-rbac')
      .with_notify('Service[pe-console-services]') }
    it { should_not contain_pe_hocon_setting('console-services.rbac.database.password') }
    it { should contain_pe_hocon_setting('console-services.rbac.database.maximum-pool-size')
      .with_value(10)
      .with_notify('Service[pe-console-services]') }
    it { should contain_pe_hocon_setting('console-services.rbac.database.connection-timeout')
      .with_value(60000)
      .with_notify('Service[pe-console-services]') }
    it { should contain_pe_hocon_setting('console-services.rbac.database.connection-check-timeout')
      .with_value(5000)
      .with_notify('Service[pe-console-services]') }
  end

  context "customizing parameters" do
    before(:each) do
      params.merge!({ 'database_host'      => 'random.rspec',
                      'database_port'      => 1234,
                      'database_user'      => 'test-user',
                      'database_password'  => 'test_password',
                      'migration_user'     => 'migration_user',
                      'migration_password' => 'migration_password',
                      'database_name'      => 'test-db',
                    })
    end

    it { should contain_pe_hocon_setting('console-services.rbac.database.subname')
      .with_value('//random.rspec:1234/test-db')
      .with_notify('Service[pe-console-services]') }
    it { should contain_pe_hocon_setting('console-services.rbac.database.user')
      .with_value('test-user')
      .with_notify('Service[pe-console-services]') }
    it { should contain_pe_hocon_setting('console-services.rbac.database.password')
      .with_value('test_password')
      .with_notify('Service[pe-console-services]') }
    it { should contain_pe_hocon_setting('console-services.rbac.database.maximum-pool-size')
      .with_value(10)
      .with_notify('Service[pe-console-services]') }
    it { should contain_pe_hocon_setting('console-services.rbac.database.connection-timeout')
      .with_value(60000)
      .with_notify('Service[pe-console-services]') }
    it { should contain_pe_hocon_setting('console-services.rbac.database.connection-check-timeout')
      .with_value(5000)
      .with_notify('Service[pe-console-services]') }
  end
end
