require 'spec_helper'

describe 'puppet_enterprise::trapperkeeper::global_settings' do
  let(:params) { {} }
  let(:node) { 'master.rspec' }
  let(:title) { 'orchestration-services' }
  let(:service) { 'Service[pe-orchestration-services]' }

  let(:pre_condition) do
    <<-PRE_COND
      class {'puppet_enterprise':
        puppet_master_host           => 'master.rspec',
      }
      service { 'pe-orchestration-services': }
    PRE_COND
  end

  context "managing the global config file" do
    it { should contain_file("/etc/puppetlabs/orchestration-services/conf.d/global.conf")
      .with_ensure('file')
      .with_mode('0640')
      .with_owner('pe-orchestration-services')
      .with_group('pe-orchestration-services')}

    it { should contain_pe_hocon_setting("orchestration-services.global.logging-config")
      .with_setting("global.logging-config")
      .with_value("/etc/puppetlabs/orchestration-services/logback.xml").with_notify(service) }
  end

  context "when managing global.hostname" do
    before(:each) do
      params['hostname'] = 'test.blah.com'
    end
    it { should contain_pe_hocon_setting("orchestration-services.global.hostname")
      .with_setting("global.hostname")
      .with_value("test.blah.com").with_notify(service) }
  end
  context "when not managing global.hostname" do
    before(:each) do
      params['hostname'] = :undef
    end
    it { should contain_pe_hocon_setting("orchestration-services.global.hostname")
      .with_ensure('absent') }
  end

  context "when managing global.ssl-cert" do
    before(:each) do
      params['ssl_cert'] = '/etc/puppetlabs/orchestration-services/ssl/master.rspec.cert.pem'
    end
    it { should contain_pe_hocon_setting("orchestration-services.global.certs.ssl-cert")
      .with_setting("global.certs.ssl-cert")
      .with_value("/etc/puppetlabs/orchestration-services/ssl/master.rspec.cert.pem").with_notify(service) }
  end
  context "when not managing global.ssl-cert" do
    before(:each) do
      params['ssl_cert'] = :undef
    end
    it { should contain_pe_hocon_setting("orchestration-services.global.certs.ssl-cert")
      .with_ensure('absent') }
  end

  context "when managing global.ssl-key" do
    before(:each) do
      params['ssl_key'] = "/etc/puppetlabs/orchestration-services/ssl/master.rspec.private_key.pem"
    end
    it { should contain_pe_hocon_setting("orchestration-services.global.certs.ssl-key")
      .with_setting("global.certs.ssl-key")
      .with_value("/etc/puppetlabs/orchestration-services/ssl/master.rspec.private_key.pem").with_notify(service) }
  end
  context "when not managing global.ssl-key" do
    before(:each) do
      params['ssl_key'] = :undef
    end
    it { should contain_pe_hocon_setting("orchestration-services.global.certs.ssl-key")
      .with_ensure('absent') }
  end

  context "when managing global.ssl-ca-cert" do
    before(:each) do
      params['localcacert'] = "/etc/puppetlabs/puppet/ssl/certs/ca.pem"
    end
    it { should contain_pe_hocon_setting("orchestration-services.global.certs.ssl-ca-cert")
      .with_setting("global.certs.ssl-ca-cert")
      .with_value("/etc/puppetlabs/puppet/ssl/certs/ca.pem").with_notify(service) }
    end
  context "when not managing global.ssl-ca-cert" do
    before(:each) do
      params['localcacert'] = :undef
    end
    it { should contain_pe_hocon_setting("orchestration-services.global.certs.ssl-ca-cert")
      .with_ensure('absent') }
  end

  context "when managing global.login-path" do
    before(:each) do
      params['login_path'] = '/auth/login'
    end
    it { should contain_pe_hocon_setting('orchestration-services.global.login-path')
      .with_setting('global.login-path')
      .with_value('/auth/login').with_notify(service) }
    end
  context "when not managing global.login-path" do
    before(:each) do
      params['login_path'] = :undef
    end
    it { should contain_pe_hocon_setting('orchestration-services.global.login-path')
      .with_ensure('absent') }
  end

  context "when managing global.version-path" do
    before(:each) do
      params['version_path'] = '/opt/puppetlabs/server/pe_version'
    end
    it { should contain_pe_hocon_setting('orchestration-services.global.version-path')
      .with_setting('global.version-path')
      .with_value('/opt/puppetlabs/server/pe_version').with_notify(service) }
    end
  context "when not managing global.version-path" do
    before(:each) do
      params['version_path'] = :undef
    end
    it { should contain_pe_hocon_setting('orchestration-services.global.version-path')
      .with_ensure('absent') }
  end

  context "when managing global.replication-mode" do
    before(:each) do
      params['replication_mode'] = 'none'
    end
    it { should contain_pe_hocon_setting('orchestration-services.global.replication-mode')
      .with_setting('global.replication-mode')
      .with_value('none').with_notify(service) }
    end
  context "when not managing global.replication-mode" do
    before(:each) do
      params['replication_mode'] = :undef
    end
    it { should contain_pe_hocon_setting('orchestration-services.global.replication-mode')
      .with_ensure('absent') }
  end

  context "when setting service user and group" do
    before(:each) do
      params['service_user'] = 'pe-puppet'
      params['service_group'] = 'pe-puppet'
    end
    it { should contain_file("/etc/puppetlabs/orchestration-services/conf.d/global.conf")
      .with_ensure('file')
      .with_mode('0640')
      .with_owner('pe-puppet')
      .with_group('pe-puppet')}
  end
end
