require 'spec_helper'

describe 'puppet_enterprise::trapperkeeper::init_defaults' do
  let(:facts) do
    {
      :pe_build => '2017.2.0',
    }
  end
  let(:params) { {} }
  let(:title) { 'puppetserver' }
  let(:package) { 'Package[pe-puppetserver]' }
  let(:full_restart) { 'Exec[pe-puppetserver service full restart]'}

  let(:pre_condition) do
    <<-PRE_COND
      class {'puppet_enterprise':
        puppet_master_host           => 'master.rspec',
      }
      package { 'pe-puppetserver': }
      puppet_enterprise::trapperkeeper::pe_service { 'puppetserver': }
    PRE_COND
  end

  context "managing service defaults" do
    it { should contain_pe_ini_setting('puppetserver initconf java_bin')
         .with_setting('JAVA_BIN')
         .with_value('"/opt/puppetlabs/server/bin/java"')
         .with_path('/etc/sysconfig/pe-puppetserver')
         .with_notify(full_restart)
         .with_require(package) }
    it { should contain_pe_ini_setting('puppetserver initconf user')
         .with_setting('USER')
         .with_value('pe-puppetserver')
         .with_notify(full_restart)
         .with_require(package) }
    it { should contain_pe_ini_setting('puppetserver initconf group')
         .with_setting('GROUP')
         .with_value('pe-puppetserver')
         .with_notify(full_restart)
         .with_require(package) }
    it { should contain_pe_ini_setting('puppetserver initconf install_dir')
         .with_setting('INSTALL_DIR')
         .with_value('"/opt/puppetlabs/server/apps/puppetserver"')
         .with_notify(full_restart)
         .with_require(package) }
    it { should contain_pe_ini_setting('puppetserver initconf config')
         .with_setting('CONFIG')
         .with_value('"/etc/puppetlabs/puppetserver/conf.d"')
         .with_notify(full_restart)
         .with_require(package) }
    it { should contain_pe_ini_setting('puppetserver initconf bootstrap_config')
         .with_setting('BOOTSTRAP_CONFIG')
         .with_value('"/etc/puppetlabs/puppetserver/bootstrap.cfg"')
         .with_notify(full_restart)
         .with_require(package) }
    it { should contain_pe_ini_setting('puppetserver initconf service_stop_retries')
         .with_setting('SERVICE_STOP_RETRIES')
         .with_value('60')
         .with_notify(full_restart)
         .with_require(package) }
    it { should contain_pe_ini_setting('puppetserver initconf start_timeout')
         .with_setting('START_TIMEOUT')
         .with_value('300')
         .with_notify(full_restart)
         .with_require(package) }
    it { should contain_pe_ini_setting('puppetserver initconf jruby_jar')
         .with_ensure('absent') }

    context "with overrides" do
      before(:each) do
        params['service_stop_retries'] = 12345
        params['start_timeout'] = 67890
        params['user'] = 'pe-puppet'
        params['group'] = 'pe-puppet'
        params['jruby_jar'] = 'foo'
      end

      it { should contain_pe_ini_setting('puppetserver initconf service_stop_retries')
           .with_setting('SERVICE_STOP_RETRIES')
           .with_value('12345')
           .with_notify(full_restart)
           .with_require(package) }
      it { should contain_pe_ini_setting('puppetserver initconf start_timeout')
           .with_setting('START_TIMEOUT')
           .with_value('67890')
           .with_notify(full_restart)
           .with_require(package) }
      it { should contain_pe_ini_setting('puppetserver initconf user')
          .with_setting('USER')
          .with_value('pe-puppet')
          .with_notify(full_restart)
          .with_require(package) }
      it { should contain_pe_ini_setting('puppetserver initconf group')
          .with_setting('GROUP')
          .with_value('pe-puppet')
          .with_notify(full_restart)
          .with_require(package) }
      it { should contain_pe_ini_setting('puppetserver initconf jruby_jar')
          .with_setting('JRUBY_JAR')
          .with_value('"foo"')
          .with_notify(full_restart)
          .with_require(package) }
    end
  end
end
