require 'spec_helper'

describe 'puppet_enterprise::trapperkeeper::java_args' do
  let(:facts) { {} }
  let(:params) do
    {
      'java_args' => {
        'Xmx' => '345m',
        'Xms' => '123m',
      }
    }
  end
  let(:title) { 'puppetdb' }

  let(:pre_condition) do
    <<-PRE_COND
      class {'puppet_enterprise':
        puppet_master_host           => 'master.rspec',
      }
      package { 'pe-puppetdb': }
      puppet_enterprise::trapperkeeper::pe_service { 'puppetdb': }
    PRE_COND
  end

  context "default" do

    it { should contain_pe_ini_subsetting("pe-puppetdb_'Xmx'").with(
     'value'   => '345m',
     'require' => 'Package[pe-puppetdb]',
    ) }
    it { should contain_pe_ini_subsetting("pe-puppetdb_'Xms'").with(
     'value'   => '123m',
     'require' => 'Package[pe-puppetdb]',
    ) }

  end

  context "when enabling gc_logging" do
    before(:each) do
      params['enable_gc_logging'] = true
    end

    it { should contain_pe_ini_subsetting("pe-puppetdb_'XX:+PrintGCDetails'").with(
     'value'   => '',
     'require' => 'Package[pe-puppetdb]',
    ) }
    it { should contain_pe_ini_subsetting("pe-puppetdb_'XX:+PrintGCDateStamps'").with(
     'value'   => '',
     'require' => 'Package[pe-puppetdb]',
    ) }
    it { should contain_pe_ini_subsetting("pe-puppetdb_'Xloggc:'").with(
     'value'   => '/var/log/puppetlabs/puppetdb/puppetdb_gc.log',
     'require' => 'Package[pe-puppetdb]',
    ) }
    it { should contain_pe_ini_subsetting("pe-puppetdb_'XX:+UseGCLogFileRotation'").with(
     'value'   => '',
     'require' => 'Package[pe-puppetdb]',
    ) }
    it { should contain_pe_ini_subsetting("pe-puppetdb_'XX:NumberOfGCLogFiles='").with(
     'value'   => '16',
     'require' => 'Package[pe-puppetdb]',
    ) }
    it { should contain_pe_ini_subsetting("pe-puppetdb_'XX:GCLogFileSize='").with(
     'value'   => '64m',
     'require' => 'Package[pe-puppetdb]',
    ) }
  end

  context "when disabling gc_logging" do
    before(:each) do
      params['enable_gc_logging'] = false
    end

    it { should contain_pe_ini_subsetting("pe-puppetdb_'XX:+PrintGCDetails'").with_ensure('absent') }
    it { should contain_pe_ini_subsetting("pe-puppetdb_'XX:+PrintGCDateStamps'").with_ensure('absent') }
    it { should contain_pe_ini_subsetting("pe-puppetdb_'Xloggc:'").with_ensure('absent') }
    it { should contain_pe_ini_subsetting("pe-puppetdb_'XX:+UseGCLogFileRotation'").with_ensure('absent') }
    it { should contain_pe_ini_subsetting("pe-puppetdb_'XX:NumberOfGCLogFiles='").with_ensure('absent') }
    it { should contain_pe_ini_subsetting("pe-puppetdb_'XX:GCLogFileSize='").with_ensure('absent') }
  end

  context "when on version above the LTS" do
    before(:each) do
      facts['pe_build'] = '2017.1.1'
    end

    it { should contain_pe_ini_subsetting("pe-puppetdb_'Xmx'").with(
      'notify' => 'Exec[pe-puppetdb service full restart]',
    ) }
  end
  context "when on the LTS" do
    before(:each) do
      facts['pe_build'] = '2016.4.4'
    end

    it { should contain_pe_ini_subsetting("pe-puppetdb_'Xmx'").with(
      'notify' => 'Service[pe-puppetdb]',
    ) }
  end
  context "when on version above the LTS and setting up a compile master" do
    before(:each) do
      # Update facts so the the updated function is recognized for subsequent tests.
      # I think it's because rspec-puppet has an @@cache based on args, and
      # facts are one of the arguments.  So by mutating facts, we are forcing
      # it get a new catalog for evaluation and the overridden function is  used...
      facts['foo'] = 'baz'
      facts['pe_build'] = nil
      Puppet::Parser::Functions.newfunction(:pe_compiling_server_version, :type => :rvalue) do |args|
        '2017.1.1'
      end
      Puppet::Parser::Functions.newfunction(:pe_compile_master, :type => :rvalue) do |args|
        true
      end
    end

    it { should contain_pe_ini_subsetting("pe-puppetdb_'Xmx'").with(
      'notify' => 'Exec[pe-puppetdb service full restart]',
    ) }
  end
  context "when on the LTS and setting up a compile master" do
    before(:each) do
     # Update facts so the the updated function is recognized for subsequent tests.
     # I think it's because rspec-puppet has an @@cache based on args, and
     # facts are one of the arguments.  So by mutating facts, we are forcing
     # it get a new catalog for evaluation and the overridden function is  used...
      facts['foo'] = 'bar'
      facts['pe_build'] = nil
      Puppet::Parser::Functions.newfunction(:pe_compiling_server_version, :type => :rvalue) do |args|
        '2016.4.4'
      end
      Puppet::Parser::Functions.newfunction(:pe_compile_master, :type => :rvalue) do |args|
        true
      end
    end

    it { should contain_pe_ini_subsetting("pe-puppetdb_'Xmx'").with(
      'notify' => 'Service[pe-puppetdb]',
    ) }
  end
end
