require 'spec_helper'

describe 'puppet_enterprise::trapperkeeper::pcp_broker' do
  let(:title) { 'orchestration-services' }
  let(:config_dir) { "/etc/puppetlabs/#{title}" }

  let(:pre_condition) do
    <<-PRE_COND
      class {'puppet_enterprise':
        puppet_master_host           => 'master.rspec',
        console_host                 => 'console.rspec',
      }
      service { 'pe-orchestration-services': }
      package { 'pe-orchestration-services': }
    PRE_COND
  end

  context "configuring the pcp-broker" do
    let(:params) { {'controller_uris' => ['orchestrator.rspec']} }

    it { should contain_pe_hocon_setting("#{title}.pcp-broker.accept-consumers").with_ensure('absent') }
    it { should contain_pe_hocon_setting("#{title}.pcp-broker.delivery-consumers").with_ensure('absent') }
    it { should contain_pe_hocon_setting("#{title}.pcp-broker.controller-uris").with_value(['orchestrator.rspec']) }
    it { should contain_pe_hocon_setting("#{title}.pcp-broker.controller-whitelist")
      .with_type('array')
      .with_value([
        "http://puppetlabs.com/inventory_request",
        "http://puppetlabs.com/rpc_blocking_request",
        "http://puppetlabs.com/rpc_non_blocking_request",
      ])}
  end

  context "configuring authorization" do
    it { should contain_pe_puppet_authorization__rule('pxp commands')
      .with_match_request_path('/pcp-broker/send')
      .with_match_request_type('path')
      .with_match_request_query_params({'message_type' => [
        'http://puppetlabs.com/rpc_non_blocking_request',
        'http://puppetlabs.com/rpc_blocking_request']})
      .with_allow([
        'console.rspec',
        'master.rspec'])
      .with_path("#{config_dir}/conf.d/auth.conf")
      .with_sort_order(400) }
    it { should contain_pe_puppet_authorization__rule('inventory request')
      .with_match_request_path('/pcp-broker/send')
      .with_match_request_type('path')
      .with_match_request_query_params({'message_type' => [
        'http://puppetlabs.com/inventory_request']})
      .with_allow([
        'console.rspec',
        'master.rspec'])
      .with_path("#{config_dir}/conf.d/auth.conf")
      .with_sort_order(400) }
    it { should contain_pe_puppet_authorization__rule('multi-cast with destination_report')
      .with_match_request_path('/pcp-broker/send')
      .with_match_request_type('path')
      .with_match_request_query_params({'targets' => [
        'pcp://*/agent',
        'pcp://*/*'],
        'destination_report' => 'true'})
      .with_allow([])
      .with_path("#{config_dir}/conf.d/auth.conf")
      .with_sort_order(399) }
    it { should contain_pe_puppet_authorization__rule('pcp-broker message')
      .with_match_request_path('/pcp-broker/send')
      .with_match_request_type('path')
      .with_match_request_query_params({'message_type' => [
        'http://puppetlabs.com/associate_request',
        'http://puppetlabs.com/rpc_provisional_response',
        'http://puppetlabs.com/rpc_blocking_response',
        'http://puppetlabs.com/rpc_non_blocking_response',
        'http://puppetlabs.com/rpc_error_message']})
      .with_allow('*')
      .with_path("#{config_dir}/conf.d/auth.conf")
      .with_sort_order(420) }
    it { should contain_pe_puppet_authorization__rule('pcp messages')
      .with_ensure('absent')
      .with_path("#{config_dir}/conf.d/auth.conf") }
  end

  context "configuring bootstrap.cfg" do
    it { should contain_pe_concat__fragment('orchestration-services broker-service') }
    it { should contain_pe_concat__fragment('orchestration-services authorization-service') }
    it { should contain_pe_concat__fragment('orchestration-services jetty9-service') }
    it { should contain_pe_concat__fragment('orchestration-services webrouting-service') }
  end
end
