#! /usr/bin/env ruby -S rspec

require 'spec_helper'

describe "interval_and_offset_to_cron_time" do

  it "should exist as named" do
    expect(Puppet::Parser::Functions.function("interval_and_offset_to_cron_time")).to eq("function_interval_and_offset_to_cron_time")
  end

  # Arguments

  context "when less than two parameters are given" do
    it { is_expected.to run.with_params().and_raise_error(
        Puppet::ParseError, /wrong number of arguments given/
      )
    }
  end

  context "when more than two parameters are given" do
    it { is_expected.to run.with_params(1, 2, 3).and_raise_error(
        Puppet::ParseError, /wrong number of arguments given/
      )
    }
  end
  
  context "when interval is less than 0" do
    it { is_expected.to run.with_params(-1, -1).and_raise_error(
        Puppet::ParseError, /interval must be between 0 and 1440/
      )
    }
  end

  context "when interval is greater than 1440" do
    it { is_expected.to run.with_params(1441, 1).and_raise_error(
        Puppet::ParseError, /interval must be between 0 and 1440/
      )
    }
  end

  context "when offset is less than 0" do
    it { is_expected.to run.with_params(60, -1).and_raise_error(
        Puppet::ParseError, /offset must be between 0 and 59/
      )
    }
  end

  context "when offset is greater than 59" do
    it { is_expected.to run.with_params(60, 60).and_raise_error(
        Puppet::ParseError, /offset must be between 0 and 59/
      )
    }
  end

  context "when offset is greater than interval" do
    it { is_expected.to run.with_params(15, 30).and_raise_error(
        Puppet::ParseError, /offset must be less than or equal to interval/
      )
    }
  end

  # Results

  context "when interval is equal to 0" do
    it { is_expected.to run.with_params(0, 0).and_return({
        'hour'   => 'absent',
        'minute' => 'absent',
      })
    }
  end

  context "when interval is equal to 1" do
    it { is_expected.to run.with_params(1, 1).and_return({
        'hour'   => '*',
        'minute' => '*',
      })
    }
  end

  context "when interval is less than 30" do
    it { is_expected.to run.with_params(15, 1).and_return({
        'hour'   => '*',
        'minute' => [1, 16, 31, 46],
      })
    }
  end

  context "when interval is equal to 30" do
    it { is_expected.to run.with_params(30, 1).and_return({
        'hour'   => '*',
        'minute' => [1, 31],
      })
    }
  end

  context "when interval is more than 30 and less than 60" do
    it { is_expected.to run.with_params(45, 1).and_return({
        'hour'   => '*',
        'minute' => 1,
      })
    }
  end

  context "when interval is equal to 60" do
    it { is_expected.to run.with_params(60, 1).and_return({
        'hour'   => '*',
        'minute' => 1,
      })
    }
  end

  context "when interval is greater than 60" do
    it { is_expected.to run.with_params(75, 1).and_return({
        'hour'   => '*/2',
        'minute' => 1,
      })
    }
  end

end