#! /usr/bin/env ruby -S rspec
require 'spec_helper'

describe "pe_format_urls" do
  it "should exist" do
    expect(Puppet::Parser::Functions.function("pe_format_urls")).to eq("function_pe_format_urls")
  end

  it "should raise a ParseError if there are fewer than 3 arguments" do
    expect(subject).to run.with_params(1,2).and_raise_error(ArgumentError)
  end

  it "should raise a ParseError if there are more than 3 arguments" do
    expect(subject).to run.with_params(1,2,3,4).and_raise_error(ArgumentError)
  end

  it "should return correctly" do
    expect(subject).to run.with_params('https', ['puppetdb.rspec'], ['8081']).and_return(['https://puppetdb.rspec:8081'])
  end

  it "should return correctly 2" do
    expect(subject).to run.with_params('https', ['puppetdb.rspec', 'secondary_puppetdb.rspec'], ['8081']).and_return(['https://puppetdb.rspec:8081','https://secondary_puppetdb.rspec:8081'])
  end

  it "should return correctly 3" do
    expect(subject).to run.with_params('https', ['puppetdb.rspec', 'secondary_puppetdb.rspec'], ['8081','7777']).and_return(['https://puppetdb.rspec:8081','https://secondary_puppetdb.rspec:7777'])
  end

  it "should return correctly 4" do
    expect(subject).to run.with_params('https',
                                      ['puppetdb.rspec',
                                       'secondary_puppetdb.rspec',
                                       'third_puppetdb.rspec'],
                                       ['8081','7777','6666']).and_return(
                                                              ['https://puppetdb.rspec:8081',
                                                               'https://secondary_puppetdb.rspec:7777',
                                                               'https://third_puppetdb.rspec:6666'])
  end
end
