#! /usr/bin/env ruby -S rspec
require 'spec_helper'

describe 'pe_hash_array_sort' do
  it 'should exist' do
    expect(Puppet::Parser::Functions.function('pe_hash_array_sort')).to eq('function_pe_hash_array_sort')
  end

  it 'should raise a ParseError if there are not 2 arguments' do
    expect(subject).to run.with_params().and_raise_error(Puppet::ParseError)
    expect(subject).to run.with_params([]).and_raise_error(Puppet::ParseError)
    expect(subject).to run.with_params([], 'param1', 'param2').and_raise_error(Puppet::ParseError)
  end

  it 'should raise a ParseError if there are not Hashes contained in the Array to sort' do
    expect(subject).to run.with_params(['not a hash!', 'nope!']).and_raise_error(Puppet::ParseError)
  end

  it 'should allow empty Array' do
    expect(subject).to run.with_params([], 'key').and_return([])
  end

  it 'should not sort if keys provided do not exist in the Hashes' do
    expect(subject).to run.with_params([{'key' => 'a'}, {'key' => 'c'}, {'key' => 'b'}], 'dontexist').and_return([{'key' => 'a'}, {'key' => 'c'}, {'key' => 'b'}])
    expect(subject).to run.with_params([{'key' => 'a'}, {'key' => 'c'}, {'key' => 'b'}], ['dontexist']).and_return([{'key' => 'a'}, {'key' => 'c'}, {'key' => 'b'}])
    expect(subject).to run.with_params([{'key' => 'a'}, {'key' => 'c'}, {'key' => 'b'}], ['dontexist', 'dontexisteither']).and_return([{'key' => 'a'}, {'key' => 'c'}, {'key' => 'b'}])
    expect(subject).to run.with_params([{'key' => 'a'}, {'key' => 'c'}, {'key' => 'b'}], []).and_return([{'key' => 'a'}, {'key' => 'c'}, {'key' => 'b'}])
  end

  it 'should sort Array of Hashes by supplied Hash key' do
    expect(subject).to run.with_params([{'key' => 'a'}, {'key' => 'c'}, {'key' => 'b'}], 'key').and_return([{'key' => 'a'}, {'key' => 'b'}, {'key' => 'c'}])
    expect(subject).to run.with_params([{'key' => 'a'}, {'key' => 'c'}, {'key' => 'b'}], ['key']).and_return([{'key' => 'a'}, {'key' => 'b'}, {'key' => 'c'}])
  end

  it 'should sort Array of Hashes by supplied Array of Hash keys' do
    expect(subject).to run.with_params([{'key1' => 'a', 'key2' => 'b'},{'key1' => 'a', 'key2' => 'a'}, {'key1' => 'b', 'key2' => 'a'}], ['key1', 'key2']).and_return([{'key1' => 'a', 'key2' => 'a'}, {'key1' => 'a', 'key2' => 'b'}, {'key1' => 'b', 'key2' => 'a'}])
    expect(subject).to run.with_params([{'key1' => 'a', 'key2' => 'b'}, {'key1' => 'a', 'key2' => 'a'}, {'key1' => 'b', 'key2' => 'a'}], ['key2', 'key1']).and_return([{'key1' => 'a', 'key2' => 'a'}, {'key1' => 'b', 'key2' => 'a'}, {'key1' => 'a', 'key2' => 'b'}])
  end
end