#! /usr/bin/env ruby -S rspec
require 'spec_helper'

describe "puppet_enterprise::on_the_lts" do
  let(:facts) { {} }

  context "when using on_the_lts on 2016.4 not on a compile master" do
    before :each do
      facts['foo'] = '1'
      facts['pe_build'] = '2016.4.4'
      Puppet::Parser::Functions.newfunction(:pe_compile_master, :type => :rvalue) do |args|
        false
      end
      Puppet::Parser::Functions.newfunction(:pe_compiling_server_version, :type => :rvalue) do |args|
        '2017.1.1'
      end
    end
    it { is_expected.to run.and_return(true) }
  end

  context "when using on_the_lts on 2016.4 and on a compile master" do
    before :each do
      facts['foo'] = '2'
      facts['pe_build'] = '2016.2.1'
      Puppet::Parser::Functions.newfunction(:pe_compile_master, :type => :rvalue) do |args|
        true
      end
      Puppet::Parser::Functions.newfunction(:pe_compiling_server_version, :type => :rvalue) do |args|
        '2016.4.4'
      end
    end
    it { is_expected.to run.and_return(true) }
  end

  context "when using on_the_lts on non-lts-version not on a compile master" do
    before :each do
      facts['foo'] = '3'
      facts['pe_build'] = '2017.1.1'
      Puppet::Parser::Functions.newfunction(:pe_compile_master, :type => :rvalue) do |args|
        false
      end
      Puppet::Parser::Functions.newfunction(:pe_compiling_server_version, :type => :rvalue) do |args|
        '2016.4.4'
      end
    end
    it { is_expected.to run.and_return(false) }
  end

  context "when using on_the_lts on non-lts-version and on a compile master" do
    before :each do
      facts['foo'] = '4'
      facts['pe_build'] = nil
      Puppet::Parser::Functions.newfunction(:pe_compile_master, :type => :rvalue) do |args|
        true
      end
      Puppet::Parser::Functions.newfunction(:pe_compiling_server_version, :type => :rvalue) do |args|
        '2017.1.1'
      end
    end
    it { is_expected.to run.and_return(false) }
  end
end
