require 'spec_helper'

describe 'platform_tag fact' do
  let(:os) do
    _os = {
      'family'       => osfamily,
      'name'         => osname,
      'release'      => {
        'full'  => full_version,
        'major' => major_version,
      }
    }
    _os['release']['minor'] = minor_version if !minor_version.nil?
    _os['architecture'] = arch if !arch.nil?
    _os
  end
  let(:minor_version) { nil }

  before :each do
    Facter.reset

    _os = os
    Facter.add(:os) do
      setcode do
        _os
      end
    end
    # Platform architecture is not reset by os value
    _arch = arch
    Facter.add(:architecture) do
      setcode { _arch }
    end
  end

  context 'Redhat' do
    let(:osfamily) { 'RedHat' }
    let(:osname) { 'RedHat' }
    let(:arch) { 'x86_64' }
    let(:full_version) { '7.2.1111' }
    let(:major_version) { '7' }

    it { expect(Facter.value('platform_tag')).to eq('el-7-x86_64') }

    context 'Fedora' do
      let(:osname) { 'Fedora' }
      let(:full_version) { '25' }
      let(:major_version) { '23' }
      let(:arch) { 'i386' }

      it { expect(Facter.value('platform_tag')).to eq('fedora-23-i386') }
    end

    context 'CentOS' do
      let(:osname) { 'CentOS' }
      let(:full_version) { '6.1.1111' }
      let(:major_version) { '6' }

      it { expect(Facter.value('platform_tag')).to eq('el-6-x86_64') }
    end

    context 'Amazon' do
      let(:osname) { 'Amazon' }

      context '2017.3' do
        let(:full_version) { '2017.3' }
        let(:major_version) { '2017' }
        let(:minor_version) { '3' }

        it { expect(Facter.value('platform_tag')).to eq('el-6-x86_64') }
      end

      context '2017.12' do
        let(:full_version) { '2017.12' }
        let(:major_version) { '2017' }
        let(:minor_version) { '12' }

        it { expect(Facter.value('platform_tag')).to eq('el-7-x86_64') }
      end

      context '2' do
        let(:full_version) { '2' }
        let(:major_version) { '2' }

        it { expect(Facter.value('platform_tag')).to eq('el-7-x86_64') }
      end

      context '2.x' do
        let(:full_version) { '2.1' }
        let(:major_version) { '2' }
        let(:minor_version) { '1' }

        it { expect(Facter.value('platform_tag')).to eq('el-7-x86_64') }
      end
    end
  end

  context 'Debian' do
    let(:osfamily) { 'Debian' }
    let(:osname) { 'Debian' }
    let(:arch) { 'amd64' }
    let(:full_version) { '8.0' }
    let(:major_version) { '8' }

    it { expect(Facter.value('platform_tag')).to eq('debian-8-amd64') }

    context 'Ubuntu' do
      let(:osname) { 'Ubuntu' }
      let(:arch) { 'i386' }
      let(:full_version) { '16.04' }
      let(:major_version) { '16.04' }

      it { expect(Facter.value('platform_tag')).to eq('ubuntu-16.04-i386') }

      context 'ppc' do
        let(:arch) { 'ppc64le' }

        it { expect(Facter.value('platform_tag')).to eq('ubuntu-16.04-ppc64el') }
      end
    end
  end

  context 'Darwin' do
    let(:osfamily) { 'Darwin' }
    let(:osname) { 'Darwin' }
    let(:arch) { 'x86_64' }
    let(:full_version) { '16.0.0' }
    let(:major_version) { '16' }
    before(:each) do
      # The macosx_* facts don't seem to get autocreated based on os fact...
      Facter.add(:macosx_productversion_major) do
        setcode {'10.12' }
      end
    end

    it { expect(Facter.value('platform_tag')).to eq('osx-10.12-x86_64') }
  end

  context 'AIX' do
    let(:osfamily) { 'AIX' }
    let(:osname) { 'AIX' }
    let(:arch) { nil }
    let(:full_version) { '6100-07-04-1216' }
    let(:major_version) { '6100' }
    before(:each) do
      _major_version = major_version
      Facter.add(:kernelrelease) do
        setcode { _major_version }
      end
    end

    it { expect(Facter.value('platform_tag')).to eq('aix-6.1-power') }
  end

  context 'Solaris' do
    let(:osfamily) { 'Solaris' }
    let(:osname) { 'Solaris' }
    let(:arch) { nil }
    let(:full_version) { '11.2' }
    let(:major_version) { '11' }
    before(:each) do
      Facter.add(:hardwareisa) do
        setcode { 'sparc' }
      end
      Facter.add(:kernelrelease) do
        setcode { '5.11' }
      end
    end

    it { expect(Facter.value('platform_tag')).to eq('solaris-11-sparc') }
  end

  context 'Sles' do
    let(:osfamily) { 'Suse' }
    let(:osname) { 'SLES' }
    let(:arch) { 'x86_64' }
    let(:full_version) { '12.1' }
    let(:major_version) { '1' }

    it { expect(Facter.value('platform_tag')).to eq('sles-12-x86_64') }
  end
end
